/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package confconv;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.File;
import java.util.Properties;
import nat.OptNames;

/**
 * Update config file to Nat-braille 2.0 format
 * @author vivien
 *
 */
public class ConfConv {
    /**
     * empty constructor 
     * does nothing
     */
    public ConfConv() {		
    }

    /**
     * 1. Liste tous les fichiers lisibles comme Properties
     *   situés dans le sous-répertoire "configurations" du répertoire
     *    dans lequel est lancé ConfConf. Ce sont les anciennes conf
     * 2. Transforme ces configs qui ne sont pas modernes (sans
     *    clé "fi-is-sys-config") en conf systeme, avec comme commentaire
     *    "configuration mise a jour" et comme nouveau nom 
     *    "config. recuperee" + l'ancien nom
     * 3. Ecrase l'ancienne conf.
     * 4. Affiche un résumé sous forme de msgbox
     * @param argv tableau des paramètres de la fonction main
     */
    public static void main (String argv []) {
		
		
		File oldConfsDir  = new File("./configurations/");		
		File[] allFiles =  oldConfsDir.listFiles();
		
		String chaineok = ""; 
		String chaineko = "";
		
		for (int i=0;i<allFiles.length;i++){
			System.out.print(allFiles[i]+"\n");			
			Properties conf = new Properties();			
			try {
				conf.load(new InputStreamReader(new FileInputStream(allFiles[i]),"UTF-8"));					
				if (conf.containsKey(OptNames.fi_is_sys_config)){
					System.out.print(" ... is a alread a modern config file \n");	
				} else {
					System.out.print(" ... is an old config file \n");					
					System.out.print(" ... adding new keys & values \n");					
					conf.setProperty(OptNames.fi_infos,"configuration mise a jour");
					conf.setProperty(OptNames.fi_name,"config. recuperee \""+allFiles[i]+"\"");
					conf.setProperty(OptNames.fi_is_sys_config,"true");
		
					System.out.print(" ... saving\n");					
					try {
						conf.store(new FileOutputStream(allFiles[i]), null);
						chaineok += allFiles[i] + "\n";
					}
					catch (IOException ioe) {
						ioe.printStackTrace(); 
						System.err.println(" ... Exception while writing filter configuration file");
						chaineko += allFiles[i] + "\n";
					}
					
				}
			} catch (IOException ioe) {
				System.out.print(" ...  not a config file \n");				
				
			}
		}
		
		// bilan
		String chaine = "";
		if (chaineok.length() != 0){
				chaine += "les conf suivantes ont ete mises a jour \n"+ chaineok +"\n";
		}
		if (chaineko.length() != 0){
				chaine += "les conf suivantes ont connu un prob lors de la maj\n"+ chaineko +"\n";
		}
		if (chaine.length() == 0){
				chaine += "aucune operation effectuee\n";
		
		}
		javax.swing.JOptionPane.showMessageDialog(null,"mise a jour des configurations \n" + chaine); 		
		

	}	

}
