/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret, 
 * Contact: bmascret@free.fr
 * 
 * Based on v2Math
 * Copyright (C) Rene Heuer,
 * http://www.exmpl.de/projects/mathml/math.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package formula;


import java.awt.Font;
import java.awt.Graphics2D;
import java.io.File;

import javax.swing.JFrame;
import javax.xml.parsers.*;  // SAXParser
import formula.display.*;			 // DISPlayOBJECTs
import formula.handlers.*;			 // DokumentHandler zur Anzeige von MathML

/**
Demonstration der einfachen Interpretation von MathML-Dokumenten.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class demo02 {

	 /**
	  * Main function
	 * @param args command line params
	 */
	public static void main( String[] args)
	{
		 MathPanel panel = new MathPanel();     // hat grafischen Kontext
		 
		 JFrame frame = new JFrame(); // das Frame
		 frame.setContentPane(panel);
		 // einfaches Frame aufbauen und anzeigen
		 frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		 frame.setTitle( "Test-Frame des Mahtematik-Displays");
		 frame.setSize( 1000, 700);
		 frame.setVisible(true);
		
		  // grafischer Kontext ist definiert. Font und Rendering setzen
		 Graphics2D g2d = (Graphics2D)panel.getGraphics();
		 g2d.setFont( new Font( "serif", java.awt.Font.PLAIN, 80));
		 g2d.setRenderingHint( java.awt.RenderingHints.KEY_ANTIALIASING, java.awt.RenderingHints.VALUE_ANTIALIAS_ON);
		
		 // Properties des/der Display-Object(s) festlegen
		 DispObject.setDefaultColor( g2d.getColor());                     // Farbe setzen
		 DispObject.setDefaultFont( g2d.getFont());                       // Font setzen
		 DispObject.setDefaultRenderContext( g2d.getFontRenderContext()); // Panel-Kontext
		 DispObject.setDefaultSpacing();								   // einfaches Spacing
		 DispObject.term = new java.io.StringWriter();                    // Ausgaben in String
		 DispObject.out  = new java.io.PrintWriter( DispObject.term);     // Ziel der Ausgabe
		
		 // MathML-Handler und -Dokument festlegen
		 MathDocHandler handler = new MathDocHandler();           // MathML-Handler
		 File   file    = new File( "formula/math1.xml"); // MathML-Dokument
		
		 // Parsen des MathML-Dokuments
		 try
		 {
			 SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
		     parser.parse( file, handler);
	     }
		 catch (Throwable throwable)
	     {
			 throwable.printStackTrace();
	         System.exit( 1);
		 }
		  
		  // Aufbau der Anzeige mit/Ohne Anzeige des Trees
		 boolean showtree = true;
		 if ( showtree) System.out.println( "MathML-Tree in postorder:\n");
		 handler.build( handler.tree.root, 0, showtree);
		
		  // Anzeigeobjekt auslesen 
		 DispObject obj = handler.type.result();
		
		 // Anzeige
		 frame.setContentPane(panel);
		 panel.setObject(obj); // Objekt an Panel binden
		 panel.repaint();    // Panel (mit Objekt) anzeigen.
		
		 // textuelle Ausgabe vorbereiten und starten
		 System.out.println( "\n\nInterne Struktur in preorder:");
		 DispObject.print( 0, obj);                       // Ausgabe erzeugen
		 System.out.println( DispObject.term.toString()); // Ausgabe anzeigen
	 }
}
