/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret, 
 * Contact: bmascret@free.fr
 * 
 * Based on v2Math
 * Copyright (C) Rene Heuer,
 * http://www.exmpl.de/projects/mathml/math.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package formula.display;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.font.*;
import java.io.*;

/**
Basisklasse aller Objekte zur grafischen Darstellung in mathematischen Formeln.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispObject
{
	 /**
	 Referenz zum Elter-Objekt.
	 */
	 public DispObject parent  = null;
	
	 /**
	 Liste mit allen Kind-Objekten.
	 */
	 public LinkedList<DispObject> objects = null;
	
	 /**
	 Darzustellendes Zeichen. Mit Space initialisiert, weil nicht darstellbar.
	 */
	 public char chr = ' ';
	
	 /**
	 Shape (Linienzug der Glyphe).
	 */
	 public Shape shp = null;
	
	 /**
	 Umgebendes Rechteck des Zeichens.
	 */
	 public Rectangle2D bnd = null;
	
	 /** 
	 Position der Grundlinie. Relativ zur oberen linken Ecke des umgebenden Rechtecks.
	 */
	 public double bse = 0;
	
	 /**
	 Farbe des Zeichens.
	 */
	 public Color col = defaultColor;
	
	 /**
	 Zeichensatz der Darstellung.
	 */
	 public Font fnt = defaultFont;
	 /**
	 Informationen zum Rendering.
	 */
	 public FontRenderContext frc = defaultContext;
	
	 /**
	 Klassenvariable mit Farbe der "normalen" Darstellung.
	 */
	 static public Color defaultColor = null;
	
	 /**
	 Klassenvariable mit Zeichensatz der "normalen" Darstellung.
	 */
	 static public Font defaultFont = null;
	
	 /**
	 Klassenvariable mit Infos zum "normalen" Rendering.
	 */
	 static public FontRenderContext defaultContext = null;
	
	 /**
	 Klassenvariable mit Abstand zwischen den Zeichen eines MathString.
	 */
	 static public double defaultCharSpace = 0.0;
	
	 /**
	 Klassenvariable mit Abstand zwischen den Elementen einer MathRow.
	 */
	 static public double defaultRowSpace = 0.0;
	
	 /**
	 Klassenvariable mit Abstand zwischen den Elementen einer MathCol.
	 */
	 static public double defaultColumnSpace = 0.0;
	 
	 /**
	 Prozentuale Verschiebung des Sub-Scriptums.
	 */
	 static public double subScript = 0.33;
	 
	 /**
	 Prozentuale Verkleinerung des Sub-Scriptums.
	 */
	 static public double subScale = 0.58;
	 
	 /**
	 Prozentuale Verschiebung des Sup-Scriptums.
	 */
	 static public double supScript = 0.33; 
	 
	 /**
	 Prozentuale Verkleinerung des Sup-Scriptums.
	 */
	 static public double supScale = 0.58;
	
	 /**
	 Writer zum Aufbau der Anzeige als Term.
	 */
	 static public StringWriter term = null; 
	
	 /**
	 PrintWriter zum einheitlichen Aufbau der Ausgaben.
	 */
	 static public PrintWriter out = null;
	
	 /**
	 Klassenvariable zur Kodierung des horizontalen Dehnens.
	 @see #scale(DispObject, double, int)
	 */
	 static public final int HORIZONTAL = 1;
	
	 /**
	 Klassenvariable zur Kodierung des vertikalen Dehnens.
	 @see #scale(DispObject, double, int)
	 */
	 static public final int VERTICAL = 2;
	
	 /**
	 Klassenvariable zur Kodierung ob Baseline angezeigt werden soll.
	 @see #paint(DispObject, int, Graphics2D)
	 @see #paintAt(DispObject, int, Graphics2D, double, double)
	 */
	 static public final int BASELINE = 1;
	 
	 /**
	 Klassenvariable zur Kodierung ob umgebendes Rechteck angezeigt werden soll.
	 @see #paint(DispObject, int, Graphics2D)
	 @see #paintAt(DispObject, int, Graphics2D, double, double)
	 */
	 static public final int BOUNDS = 2;
	
	 /**
	 Farbe für alle Objekte dieser Klasse festlegen.
	 @param color Die Farbe für Objekte dieser Klasse.
	 */
	 static public void setDefaultColor( Color color) {defaultColor = color;}
	
	 /**
	 Zeichensatz für alle Objekte dieser Klasse festlegen.
	 @param font Der Zeichensatz für Objekte dieser Klasse.
	 */
	 static public void setDefaultFont( Font font) {defaultFont = font;}
	
	 /**
	 FontRenderContext für alle Objekte dieser Klasse festlegen.
	 @param context Der FontRenderContext für Objekte dieser Klasse.
	 */
	 static public void setDefaultRenderContext( FontRenderContext context) {defaultContext = context;}
	
	 /**
	 Abstand zwischen den Zeichen einer MathString dieser Klasse festlegen.
	 Es wird ein Wert in Abhängigkeit des Zeichensatzes verwendet.
	 */
	 static public void setDefaultSpacing() 
	 {
		 DispObject mc = new DispObject();
		 defaultCharSpace = mc.fnt.getSize2D() / 24.0;
		 defaultColumnSpace = 2.0 * defaultCharSpace;
		 defaultRowSpace = 4.0 * defaultCharSpace;
	 }
	
	 /**
	 Farbe für ein bestimmtes Objekt der Klasse DispObject festlegen.
	 @param obj Objekt, welches neue Farbe erhalten soll.
	 @param color Farbe für dieses Objekt.
	 */
	 static public void setColor( DispObject obj, Color color) {obj.col = color;}
	
	 /**
	 Zeichensatz für ein bestimmtes Objekt der Klasse DispObject festlegen.
	 @param obj Objekt, welches neuen Zeichensatz erhalten soll.
	 @param font Zeichensatz für dieses Objekt.
	 */
	 static public void setFont( DispObject obj, Font font) {obj.fnt = font;}
	
	 /**
	 FontRenderContext für ein bestimmtes Objekt der Klasse DispObject festlegen.
	 @param obj Objekt, welches neuen Zeichensatz erhalten soll.
	 @param context FontRenderContext für dieses Objekt.
	 */
	 static public void setRenderContext( DispObject obj, FontRenderContext context) {obj.frc = context;}
	
	 /**
	 Signifikante Daten eines Characters für das Objekt ermitteln.
	 @param obj DispObject das den Character chr erthält.
	 @param chr Character des DispObjects.
	 //@param fnt Zeichensatz des Characters.
	 //@param frc RenderContext des Characters.
	 */
	 static public void characterData( DispObject obj, char chr)
	 {
		 obj.chr = chr;
		 String ch = ""+chr;
		 GlyphVector gly;
		 //for space chars, I don't know why 2 spaces are needed for générating only one...
		 if(chr==' '){gly=obj.fnt.createGlyphVector( obj.frc, "  ");}
		 else{gly = obj.fnt.createGlyphVector( obj.frc, ch);}
		 obj.bnd = new Rectangle2D.Double( 0.0, 0.0, gly.getVisualBounds().getWidth(), gly.getLogicalBounds().getHeight()+1);
		 obj.bse = obj.bnd.getHeight() - obj.fnt.getLineMetrics( ch, obj.frc).getDescent();
		 obj.shp = gly.getGlyphOutline( 0);
		 AffineTransform trf = new AffineTransform();
		 trf.setToTranslation( obj.bnd.getX()-gly.getVisualBounds().getX(), obj.bse);
		 obj.shp = trf.createTransformedShape( obj.shp);
	 }
	
	 /**
	 Objekte an die Objektliste anhängen.
	 Damit wird eine Elter-Kind-Beziehung hergestellt.
	 @param prt Elter-Objekt.
	 @param obj Anzuhängendes Objekt (Kind)
	 */
	 static public void append( DispObject prt, DispObject obj)
	 {
		 if (prt.objects == null) prt.objects = new LinkedList<DispObject>();
		 if (obj != null) obj.parent = prt;
		 prt.objects.add( obj);
	 }
	
	 /**
	 Bounds eines Objekts aus dem Tupel des Alignments ermitteln.
	 Diese Methode wird in vielen "build"-Methoden benutzt.
	 @param obj Objekt, dessen umgebendes Rechteck zu ermitteln ist.
	 @param tupel Array mit den ausgrichteten Rechtecken der in "obj"
	              enthaltenen Objekte.
	 */
	 static public void bounds( DispObject obj, Rectangle2D[] tupel)
	 {
		 // Bounds ermitteln
		 obj.bnd = new Rectangle2D.Double();
		 Iterator<DispObject> iter = obj.objects.iterator();
		 DispObject elm = null;
		 double     dx;
		 double     dy;
		 for (int i = 0; i < tupel.length; i++)
		 {
			 elm = (DispObject)iter.next();
			 dx = tupel[i].getX() - elm.bnd.getX();
			 dy = tupel[i].getY() - elm.bnd.getY();
			 DispObject.relPos( elm, dx, dy);
			 Rectangle2D.union( obj.bnd, elm.bnd, obj.bnd);
		 }
	 }
	
	 /**
	 Kompimieren des umgebenden Rechtecks eines Characters
	 auf den Bereich des zu zeichnenden Linienzugs. Die neuen
	 Daten des Characters werden in das DispObject übernommen.
	 Es werden nur Objekte mit vorhandenem Linienzug bearbeitet.
	 @param obj DispObject mit zu komprimierendem Character.
	 */
	 static public void compress( DispObject obj) {
	  if (obj.shp == null) return;
	  obj.bnd = obj.shp.getBounds2D();
	 AffineTransform trf = new AffineTransform();
	  trf.setToTranslation( -obj.bnd.getX(), -obj.bnd.getY());
	  obj.shp = trf.createTransformedShape( obj.shp);
	  obj.bnd = new Rectangle2D.Double( 0, 0, obj.bnd.getWidth(), obj.bnd.getHeight());
	  obj.bse = obj.bnd.getHeight();
	 }
	
	 /**
	 Skalieren des Linienzugs eines Characters. Die BaseLine liegt
	 nach Ausführung am unteren Ende des Linienzugs, genau wie
	 bei einem komprimierten Character.
	 @param obj DispObject mit dem zu skalierenden Character.
	 @param rng Bereich über den die Skalierung stattfindet.
	 @param how Angabe ob vertikale und/oder horizontale Skalierung.
	 @see #HORIZONTAL
	 @see #VERTICAL
	 */
	 static public void scale( DispObject obj, double rng, int how) {
	  if (obj.shp == null) return;
	 AffineTransform trf = new AffineTransform();
	  if ((how & HORIZONTAL) == HORIZONTAL) trf.setToScale( rng/obj.bnd.getWidth(), 1);
	  if ((how & VERTICAL)   == VERTICAL)   trf.setToScale( 1, rng/obj.bnd.getHeight());
	  obj.shp = trf.createTransformedShape( obj.shp);
	  obj.bnd = obj.shp.getBounds2D();
	  obj.bse = obj.bnd.getHeight();
	 }
	
	 /**
	 Position des Objekts und aller Sub-Objekte.
	 relativ zur aktuellen Position neu positionieren.
	 @param obj Betroffenes Objekt.
	 @param dx X-Position der Hülle.
	 @param dy Y-Position der Hülle.
	 */
	 static public void relPos( DispObject obj, double dx, double dy) {
	  if (obj == null) return;
	  obj.bnd = new Rectangle2D.Double( obj.bnd.getX()+dx, obj.bnd.getY()+dy, obj.bnd.getWidth(), obj.bnd.getHeight());
	  if (obj.objects == null) return;
	 LinkedList<DispObject> list     = obj.objects;
	 Iterator<DispObject>   iterator = list.iterator();
	  while (iterator.hasNext())
	         {
	         DispObject elm = (DispObject)iterator.next();
	          relPos( elm, dx, dy);
	         }
	 }
	
	 /**
	 Objekt und aller Sub-Objekte skalieren.
	 @param obj Betroffenes Objekt.
	 @param fct Skalierungsfacktor
	 */
	 static public void resize( DispObject obj, double fct) {
	  if (obj == null) return;
	 AffineTransform trf = new AffineTransform();
	  trf.setToScale( fct, fct);
	  obj.bse = obj.bse * fct;
	  obj.bnd = trf.createTransformedShape( obj.bnd).getBounds();
	  if (obj.shp != null) obj.shp = trf.createTransformedShape( obj.shp);
	  if (obj.objects == null) return;
	 LinkedList<DispObject> list     = obj.objects;
	 Iterator<DispObject>   iterator = list.iterator();
	  while (iterator.hasNext())
	         {
	         DispObject elm = (DispObject)iterator.next();
	          resize( elm, fct);
	         }
	 }
	
	 /**
	 Darstellung eines DispObjects.
	 @param obj Das darzustellende Object.
	 @param what Darstellung mit Grundlinie (BASELINE) und oder Rahmen (BOUNDS).
	 @see #BASELINE
	 @see #BOUNDS
	 @param g2d Der grafische Kontext für die Darstellung.
	 */
	 static public void paint( DispObject obj, int what, Graphics2D g2d) {
	 Color c = g2d.getColor();
	
	  g2d.setColor( obj.col);
	  if ((what & BOUNDS) == BOUNDS)
	     {
	      g2d.draw( obj.bnd);
	     }
	  if ((what & BASELINE) == BASELINE)
	     {
	      g2d.draw( new Line2D.Double( obj.bnd.getX(),
	                                   obj.bse+obj.bnd.getY(),
	                                   obj.bnd.getX()+obj.bnd.getWidth(),
	                                   obj.bse+obj.bnd.getY()));
	     }
	  if (obj.shp != null)
	     {
	     AffineTransform trf = new AffineTransform();
	      trf.setToTranslation( obj.bnd.getX(), obj.bnd.getY());
	      g2d.fill( trf.createTransformedShape( obj.shp));
	     }
	  g2d.setColor( c);
	  if (obj.objects != null)
	     {
	     LinkedList<DispObject> list     = obj.objects;
	     Iterator<DispObject>   iterator = list.iterator();
	      while (iterator.hasNext())
	            {
	            DispObject todraw = (DispObject)iterator.next();
	             paint( todraw, what, g2d);
	            }
	     }
	 }
	
	 /**
	 Positionierte Darstellung eines DispObjects.
	 @param obj Das darzustellende Object.
	 @param what Darstellung mit Grundlinie (BASELINE) und oder Rahmen (BOUNDS).
	 @see #BASELINE
	 @see #BOUNDS
	 @param g2d Der grafische Kontext für die Darstellung.
	 @param xpos Horizontale Position im Darstellungsbereich.
	             Bezug ist die linke Seite des umgebenden Rechtecks.
	 @param ypos Vertikale Position im Darstellungsbereich.
	             Bezug obere Seite des umgebenden Rechtecks.
	 */
	 static public void paintAt( DispObject obj, int what, Graphics2D g2d, double xpos, double ypos) {
	 if (obj == null) return;
	 AffineTransform trf = new AffineTransform();
	  trf.setToTranslation( xpos, ypos);
	 Color c = g2d.getColor();
	
	  g2d.setColor( obj.col);
	  if ((what & BOUNDS) == BOUNDS)
	     g2d.draw( trf.createTransformedShape( obj.bnd));
	  if ((what & BASELINE) == BASELINE)
	      g2d.draw( trf.createTransformedShape( new Line2D.Double( obj.bnd.getX(),
	                                                               obj.bse+obj.bnd.getY(),
	                                                               obj.bnd.getX()+obj.bnd.getWidth(),
	                                                               obj.bse+obj.bnd.getY())));
	  if (obj.shp != null)
	    {
	     trf.setToTranslation( obj.bnd.getX()+xpos, obj.bnd.getY()+ypos);
	     g2d.fill( trf.createTransformedShape( obj.shp));
	    }
	  g2d.setColor( c);
	  if (obj.objects != null)
	     {
	     LinkedList<DispObject> list     = obj.objects;
	     Iterator<DispObject>   iterator = list.iterator();
	      while (iterator.hasNext())
	            {
	            DispObject todraw = (DispObject)iterator.next();
	             paintAt( todraw, what, g2d, xpos, ypos);
	            }
	     }
	 }
	
	 /**
	 Wert eines Bytes in hexadezimaler Kodierung als String übergeben.
	 @param b Byte, dessen Hex-kodierter Wert gefragt ist.
	 @return String mit hexadezimaler Kodierung des Bytes.
	 */
	 static public String byteToHex( byte b) {
	  String hex = "0123456789ABCDEF";
	  return new String( new char[]{ hex.charAt((b >> 4) & 0x0F), hex.charAt(b & 0x0F) });
	 }
	
	 /**
	  Hexadezimale Kodierung eines Characters als String übergeben.
	  @param c Character im UTF16-Format.
	  @return String mit hexadezimaler Kodierung des Characters.
	 */
	 static public String charToHex(char c) {
	  return byteToHex((byte)(c >>> 8)) + byteToHex((byte)(c & 0xFF));
	 }
	
	 /**
	  Zeichen des Objekts als String übergeben.
	  Wenn das Zeichen nicht Buchtabe, Ziffer oder Klammer im ASCII-Bereich ist,
	  wird der entsprechende Unicode-16 in der Form "\\uXXXX" übergeben.
	  @param obj Objekt mit einzelnem Zeichen (Basisklasse).
	  @return Das Zeichen als String.
	 */
	 static public String getText( DispObject obj) {
	  if   (
	           ((Character.getType( obj.chr) & Character.LETTER_NUMBER) > 0)             // "normales" Zeichen
	        || ((Character.getType( obj.chr) & Character.INITIAL_QUOTE_PUNCTUATION) > 0) // Klammer auf
	        || ((Character.getType( obj.chr) & Character.FINAL_QUOTE_PUNCTUATION)   > 0) // Klammer zu
	       )
	    return new String( new char[]{obj.chr});
	  else return "\\u"+charToHex( obj.chr);
	 }
	
	 /**
	 Objekt rekursiv als Text ausgeben.
	 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
	 @param obj Auszugebendes Objekt.
	 */
	 static public void print( int deep, DispObject obj) {
	 Class<? extends DispObject> cls = obj.getClass();
	 java.lang.reflect.Method met = null;
	
	  DispObject.out.println();
	  for (int i=0; i<deep; i++) DispObject.out.print( " ");
	 String clsname = cls.getName();
	
	 // einfachstes Objekt: muss Character sein und hat keine Kindknoten
	 if (cls == DispObject.class)
	    {
	     DispObject.out.print( "'"+DispObject.getText( obj)+"'");
	     return;
	    }
	  
	  // Namen ausgeben
	 if (cls != DispString.class)
	     DispObject.out.print( clsname.substring( clsname.lastIndexOf( '.')+1)+"(");
	
	  // spezifische Methode zur Ausgabe suchen und ausführen
	  try   {
		     met = cls.getMethod( "printSpecial", new Class[] { int.class, DispObject.class } );
	         met.invoke( null, new Object[] {new Integer( deep), obj});
	        }
	  catch (Exception e)
	        {
	         DispObject.out.println( "Mist!");
	         e.printStackTrace();
	         //System.exit( 1);
	        }
	
	  // Rekursion beenden
	  if (cls != DispString.class)
	     {
	      DispObject.out.println();
	      for (int i=0; i<deep; i++) DispObject.out.print( " ");
	      DispObject.out.print( ")");
	     }
	 }
}
