package formula.display;

import java.util.*;
import java.awt.geom.*;

/**
Klasse zum Überstreichen eines Ausdrucks.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispOverline extends DispObject {

 /**
 Aufbau eines MathOverline-Objekts.
 @param obj Instanz eines MathOverline-Objekts, welches (neu) aufzubauen ist.
 @param chr Das Zeichen für die Überstreichung Dieses Zeichen wird um 90° in
                       mathematisch positive Richtung gedreht und dann auf die erforderliche
                       Länge gedehnt.
 @param bse Das zu überstreichende Objekt.
 */
 static public void build( DispObject obj, char chr, DispObject bse) {
 DispObject mc = new DispObject();
  DispObject.characterData( mc, chr); // Character muss "senkrecht" stehen!!!!!
  DispObject.compress( mc);
 AffineTransform trf = new AffineTransform();
  trf.setToRotation( Math.PI/2.0);                          // rotieren
  mc.shp = trf.createTransformedShape( mc.shp);
  mc.bnd = mc.shp.getBounds2D();
  trf.setToScale( bse.bnd.getWidth()/mc.bnd.getWidth(), 1); // dehnen
  mc.shp = trf.createTransformedShape( mc.shp);
  mc.bnd = mc.shp.getBounds2D();
  trf.setToTranslation( mc.bnd.getWidth(), 0);              // Korrektur der Position
  mc.shp = trf.createTransformedShape( mc.shp);
  mc.bse = mc.bnd.getHeight();
  DispOver.build( obj, bse, mc);
 }

 /**
 Spezifische Ausgabe für Objekte der MathOverline-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
 */
 static public void printSpecial( int deep, DispObject obj) {
 Iterator<DispObject>   iter = obj.objects.iterator();

  DispObject.print( deep+1, (DispObject)iter.next());
  DispObject.out.print( ",");
  DispObject.print( deep+1, (DispObject)iter.next());
 }

 /**
 Creator eines MathOverline-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see #build(DispObject, char, DispObject)
 */
 public DispOverline() {
 }

 /**
 Creator eines MathOverline-Objekts.
 @param chr Das Zeichen für die Überstreichung Dieses Zeichen wird um 90° in
            mathematisch positive Richtung gedreht und dann auf die erforderliche
            Länge gedehnt.
 @param bse Das zu überstreichende Objekt.
 */
 public DispOverline( char chr, DispObject bse) {
  build( this, chr, bse);
 }
}
