/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret, 
 * Contact: bmascret@free.fr
 * 
 * Based on v2Math
 * Copyright (C) Rene Heuer,
 * http://www.exmpl.de/projects/mathml/math.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package formula.display;

import java.util.*;
import java.awt .*;
import java.awt.geom.*;

/**
Klasse zum Aufbau zusammenhängender Zeichenkette.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DispString extends DispObject
{
	 /** DispString text color*/
	private Color color = DispObject.defaultColor;
	/** integer representing style */
	private int style = DispObject.defaultFont.getStyle();

 /**
 Farbe für ein Objekt der Klasse MathString festlegen.
 @param obj Objekt, welches neuen Zeichensatz erhalten soll.
 @param color Farbe für dieses Objekt.
 */
 static public void setColor( DispString obj, Color color) {
  obj.color = color;
 }

 /**
 Zeichensatz für ein Objekt der Klasse MathString festlegen.
 @param obj Objekt, welches neuen Zeichensatz erhalten soll.
 @param style Zeichensatz für dieses Objekt.
 */
 static public void setStyle( DispString obj, int style) {
  obj.style = style;
 }

 /**
 Aufbau eines MathString-Objekts. Es werden nur sichtbare Zeichen dargestellt,
 aber alle übernommen. Es sollte daher keine Zeilenumbrüche oder Leerzeichen
 vorhanden sein.
 @param ml Instanz eines MathString-Objekts, welches (neu) aufzubauen ist.
 @param str Die Zeichenkette des Objekts.
 */
 static public void build( DispObject ml, String str) {
	 //System.out.println(str);
  ml.objects = new LinkedList<DispObject>();
  // Liste der Objekte aufbauen
  for (int i = 0; i < str.length(); i++)
      {
      DispObject mo = new DispObject();
       DispObject.setColor( mo, ((DispString)ml).color);
       DispObject.setFont( mo, mo.fnt.deriveFont( ((DispString)ml).style));
       DispObject.characterData( mo, str.charAt(i));
       DispObject.append( ml, mo);
      }
  // Alignment der Characters
 Rectangle2D[] tupel = new Rectangle2D[str.length()]; // Align
 Iterator<DispObject> iter = ml.objects.iterator();
  for (int i = 0; i < tupel.length; i++)
        tupel[i] = ((DispObject)iter.next()).bnd;
  tupel = Align.alignRow( tupel, Align.BOTTOM, DispObject.defaultCharSpace);
  // Bounds ermitteln
  ml.bnd = new Rectangle2D.Double();
  iter = ml.objects.iterator();
 DispObject elm;
 double     dx;
 double     dy;
  for (int i = 0; i < tupel.length; i++)
       {
        elm = (DispObject)iter.next();
        dx = tupel[i].getX() - elm.bnd.getX();
        dy = tupel[i].getY() - elm.bnd.getY();
        DispObject.relPos( elm, dx, dy);
        Rectangle2D.union( ml.bnd, tupel[i], ml.bnd);
       }
  ml.bse = ((DispObject)ml.objects.getFirst()).bse; // baseline
 }

 /**
 Inhalt des Objekts als String übergeben.
 Wenn ein Zeichen nicht Buchtabe, Ziffer oder Klammer im ASCII-Bereich ist,
 wird der entsprechende Unicode-16 in der Form "\\uXXXX" eingesetzt.
 @param obj Objekt mit einzelnem Zeichen (Basisklasse).
 @return der textuelle Inhalt als String.
*/
 static public String getText( DispObject obj) {
 StringBuffer buf = new StringBuffer();
 Iterator<DispObject> iter = obj.objects.iterator();
  while (iter.hasNext()) buf.append( DispObject.getText( (DispObject)iter.next()));
  return buf.toString();
 }
 /**
 Spezifische Ausgabe für Objekte der MathString-Klasse.
 @param deep Tiefe der Einrückung. Dient der strukturierten Ausgabe.
 @param obj Objekt für textueller Ausgabe.
                       Es sollte sich unbedingt um ein MathString-Objekt handeln.
 */
 static public void printSpecial( int deep, DispObject obj) {
  DispObject.out.print( "\"");
  DispObject.out.print( getText( obj));
  DispObject.out.print( "\"");
 }

 /**
 Creator eines MathStrin-Objekts. Dieser "leere" Creator dient einer weiteren
 Flexibilisierung durch Trennung von Instanzierung und dem eigentlichen
 Aufbau des Objekts mit der Methode "build".
@see #build(DispObject, String)
 */
 public DispString() {
 }

 /**
 Creator eines MathString-Objekts.
 @param s Zeichenkette als String.
 */
 public DispString( String s) {
  build( this, s);
 }

 /**
 Creator eines MathString-Objekts mit Farb-Attribut.
 @param s Zeichenkette als String.
 @param color Farbe der Darstellung.
 */
 public DispString( String s, Color color) {
  this.color = color;
  build( this, s);
 }
 /**
 Creator eines MathString-Objekts mit Stil-Attribut.
 @param s Zeichenkette als String.
 @param style Stil der Darstellung.
 */
 public DispString( String s, int style) {
  this.style = style;
  build( this, s);
 }

 /**
 Creator eines MathString-Objekts mit Farb- und Stil-Attribut.
 @param s Zeichenkette als String.
 @param color Farbe der Darstellung.
 @param style Stil der Darstellung.
 */
 public DispString( String s, Color color, int style) {
  this.color = color;
  this.style = style;
  build( this, s);
 }
}
