/*
 * v2Math
 * Copyright (C) Rene Heuer,
 * http://www.exmpl.de/projects/mathml/math.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package formula.translators;

import formula.lists.*;	  // eigenes List/Stack-Pack
import formula.display.*; // grafische Darstellung

/**
Klasse zum Aufbau der DispObjects aus MathML-Sequenzen.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class DocToDisplay {
/**
Einfacher Stack, der in jeder Methode dieser Klasse benutzt wird.
*/
private List stack = new List(); // eigener Stack

 /**
 Komprinierung aller Elemente bis null zu einem.
 Die Elemente werden vom Stack entfernt und in einer
 neuen Liste zusammengfasst. Das null-Objekt wird ebenfalls
 vom Stack entfernt aber nicht in die Liste aufgenommen.
 @return list Liste mit allen Objekten bis zum null-Objekt.
 */
 private List buildObject() {
 List list = new List();
 Item item = List.pop( stack);
  while (item.content != null) 
	    {
	     List.push( list, item);
	     item = List.pop( stack);
	    }
  return list;
 }

 /**
 Element sichern, weil es entweder noch nicht verarbeitet wird
 oder das Ende einer Argumentliste darstellt. 
 @param obj Element als String oder null, wenn Listenende.
 */
 public void hold( Object obj) {
  List.push( stack, new Item( obj));
 }

 /**
 Einfaches String-Token behandeln.
 @param tok Name des Tokens (mi, mn, mo).
 */
 public void strtok( String tok) {
 Object obj = List.pop( stack).content; 
  List.pop( stack);
  String s;
  try{
	  s = (String)obj;
  }
  catch(ClassCastException e){s = tok;}
  List.push( stack, new Item( new DispString(s)));
 }

 /**
 Zeilen-Token (mrow) behandeln. 
 */
 public void rowtok() {
 List list = buildObject();
/*
 List list = new List();
 Item item = List.pop( stack);
  while (item.content != null) 
	    {
	     List.push( list, item);
	     item = List.pop( stack);
	    }
*/
 List arglist = new List();
  while (list.first != null) 
	    List.append( arglist, new Item( List.pop( list).content));
 DispObject[] args = new DispObject[List.length( arglist)];
  for (int i=0; i<args.length; i++)
  {
	  Object o = List.pop( arglist).content;
	  try{
		  args[i]=(DispObject)o;
	  }
	  catch(ClassCastException e){args[i]= new DispString((String)o);}
  }
  List.push( stack, new Item( new DispRow( (DispObject[])args)));
 }

 /**
 Fraction-Token behandeln.
 */
 public void frctok() {
 Object den = List.pop( stack).content; // Nenner
 Object num = List.pop( stack).content; // Zähler
  List.pop( stack); // muss NULL sein
  List.push( stack, new Item( new DispFract( (DispObject)num, (DispObject)den)));
 }

 /**
 Quadratwuzel-Token (msqrt) behandeln.
 */
 public void sqrtok() {
 Object bse = List.pop( stack).content;
  List.pop( stack); // muss NULL sein
  List.push( stack, new Item( new DispSqrt( (DispObject)bse)));
 }

 /**
 Index-Token (msub) behandeln.
 */
 public void subtok() {
 Object idx = List.pop( stack).content; // index
 Object bse = List.pop( stack).content; // basis
  List.pop( stack); // muss NULL sein
  List.push( stack, new Item( new DispMultiScript( (DispObject)bse, 
		                                           (DispObject)idx,
		                                           null, null, null)));
 }

 /**
 Exponent-Token (msup) behandeln.
 */
 public void suptok() {
 Object exp = List.pop( stack).content; // exponent
 Object bse = List.pop( stack).content; // basis
  List.pop( stack); // muss NULL sein
  List.push( stack, new Item( new DispMultiScript( (DispObject)bse, null, 
		                                           (DispObject)exp,
		                                           null, null)));
 }

 /**
 Indizierten Exponent-Token (msubsup) behandeln.
 */
 public void sbptok() {
 Object exp = List.pop( stack).content; // exponent
 Object idx = List.pop( stack).content; // index
 Object bse = List.pop( stack).content; // basis
  List.pop( stack); // muss NULL sein
  List.push( stack, new Item( new DispMultiScript( (DispObject)bse, 
		                                           (DispObject)idx,
		                                           (DispObject)exp,
		                                           null, null)));
 }

 /**
 Token zum "unterschreiben" bearbeiten.
 Es ist noch auszuprobieren wie Amaya arbeitet.
 Eigentlich sollte das Argument "mfence" hier eine Rolle spielen.  
 */
 public void udrtok() {
 DispObject from = (DispObject)List.pop( stack).content; 
 DispObject base = (DispObject)List.pop( stack).content;
 List.pop( stack); // null von munder
 DispObject.resize( from, DispObject.subScale);
 List.push( stack, new Item( new DispUnder( base, from)));  
 }

 /**
 Token zum "überschreiben" bearbeiten.
 Es ist noch auszuprobieren wie Amaya arbeitet.
 Eigentlich sollte das Argument "mfence" hier eine Rolle spielen.  
 */
 public void ovrtok() {
 DispObject to   = (DispObject)List.pop( stack).content; 
 DispObject base = (DispObject)List.pop( stack).content;
 List.pop( stack); // null von mover
 DispObject.resize( to, DispObject.supScale);
 List.push( stack, new Item( new DispOver( base, to)));  
 }

 /**
 Token zum "unter und überschreiben" bearbeiten.
 Es ist noch auszuprobieren wie Amaya arbeitet.
 Eigentlich sollte das Argument "mfence" hier eine Rolle spielen.  
 */
 public void uovtok() {
 DispObject to   = (DispObject)List.pop( stack).content; 
 DispObject from = (DispObject)List.pop( stack).content; 
 DispObject base = (DispObject)List.pop( stack).content;
 List.pop( stack); // null von munderover
 DispObject.resize( to, DispObject.supScale);
 DispObject.resize( from, DispObject.subScale);
 List.push( stack, new Item( new DispUnderover( base, from, to)));  
 }

 /**
 Klammer-Token bearbeiten.
 Es ist noch auszuprobieren wie Amaya arbeitet.
 Eigentlich sollten das Argumente und Operator-Tokens
 das gesamte Erscheinungsbild steuern.  
 */
 public void mfctok() {
 DispObject base = (DispObject)List.pop( stack).content;
 List.pop( stack); // null von mfence
 List.push( stack, new Item( new DispFence( '(', base, ')')));  
 }

 /**
 Zellen-Token (mtd) einer Tabelle (mtable) behandeln.
 Ein mrow-Token wird erzeugt, wenn mehrere Objekte vorhanden sind.
 Es wird nicht getestet, ob es innerhalb eines mrow-Tags residiert.
 */
 public void mtdtok() {
 List list = buildObject();
/*
 List list = new List();
 Item item = List.pop( stack);
  while (item.content != null) 
	    {
	     List.push( list, item);
	     item = List.pop( stack);
	    }
*/
  if (List.length( list) == 1)
     {
	  List.push( stack, List.get( list, 0));
	  return;
     }
 List arglist = new List();
  while (list.first != null) 
	    List.append( arglist, new Item( List.pop( list).content));
 DispObject[] args = new DispObject[List.length( arglist)];
  for (int i=0; i<args.length; i++)
	  args[i] = (DispObject)List.pop( arglist).content;
  List.push( stack, new Item( new DispRow( (DispObject[])args)));
 }

 /**
 Zeilen-Token (mtr) einer Tabelle (mtable) behandeln.
 Es wird nicht getestet, ob es in einem mtable-Tag residiert.
 */
 public void mtrtok() {
  List.push( stack, new Item( buildObject()));
 }

 /**
 Tabellen-Token (mtable) behandeln.
 */
 public void mtbtok() {
 List list = buildObject();
 int col = List.length( (List)list.first.content); 
 int lin = List.length( list);
 DispObject[][] arr = new DispObject[lin][col];
  for (int i=0; i<lin; i++) 
  	  {
	  List obj = (List)List.get( list, i).content;
	   for (int j=0; j<col; j++)
	        arr[i][j] = (DispObject)List.pop( obj).content;
	  }
  List.push( stack, 
		     new Item( new DispTab( arr, 
		    			            Align.CENTER, 
		    			            DispObject.defaultColumnSpace, 
		    			            Align.CENTER, 
		    			            DispObject.defaultRowSpace
		    			           )
		    		 )
		    );
 }
 
 /**
 Math-Token dient hier nur zum Test, ob MathML-Sequenz korrekt abgearbeitet wurde. 
 */
 public void mthtok() {
 Object str = List.pop( stack).content;
  // jetzt darf nur noch null-item auf dem Stack liegen.
  List.pop( stack);
  /*if (stack.first != null)
     {
	  System.out.println( "Fehler im Dokument!");
	  return;
     }*/
  List.push( stack, new Item( str));
 }

 /**
 Ergebnis übergeben.
 @return Ergebnis.   
 */
 public DispObject result() {
  return (DispObject)List.pop( stack).content;
 }
}
