/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret, 
 * Contact: bmascret@free.fr
 * 
 * Based on v2Math
 * Copyright (C) Rene Heuer,
 * http://www.exmpl.de/projects/mathml/math.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package formula.trees;

/**
Klasse eines Trees mit den Methoden als Properties.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version. 
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License. 
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die 
Free Software Foundation, Inc.,
59 Temple Place, 
Suite 330, 
Boston, MA 02111-1307, 
USA.
*/
public class Tree {
	/** erstes Element einer Liste */
	public  Node    first  = null;  // erstes Element einer Liste
	/** letztes Element einer Liste */
	public  Node    last   = null;  // letztes Element einer Liste
	/** Wurzel des Trees */
	public  Node    root   = null;  // Wurzel des Trees
	/** Indikator zur Steuerung der Erweiterung  */
	public  boolean append = false; // Indikator zur Steuerung der Erweiterung 

 /**
 Positionierung auf letztes Element der höheren Ebene.
 * @param tree the tree to be used
 */
 static public void levelUp( Tree tree) {
	 tree.first = tree.first.parent;      // Elter zetzt neuen Listenanfang
  if (tree.first == null) return; // keine Eltern vorhanden, Tree abgearbeitet
  tree.last = tree.first;              // Letzten
  while (tree.last.next != null)  // auf Listenende
	  tree.last = tree.last.next;   // positionieren
 }

 /**
 Objekt in den Tree einfügen.
 Der Indikator append steuert das Verhalten.
 Es wird unterschieden zwischen der Erstellung 
 einer neuen Kindliste (append == false) und
 einfachem Anhängen  (append == true).
 * @param tree the tree to be used
 @param object Neues Objekt im Tree.
 */
 static public void insert( Tree tree, Object object) {
 Node node = new Node( object);

  // Neuen Tree initialisieren
  if   (tree.root == null)
       {
	    tree.root  = node; // Node ist Wurzel
	    tree.first = node; // Node ist Erster
	    tree.last  = node; // Node ist Letzter
        return;
       }

  // Node an Liste (first) anhngen
  if   (tree.append)
       {
	    tree.last.next = node;              // Node ist Nachfolger
        node.prev      = tree.last;         // Node hat Vorgnger
        node.parent    = tree.first.parent; // Node hat Eltern des Ersten
        tree.last      = node;              // Node ist Letzter
        return;
       }

  // Node ist Erster in neuer Kind-Liste
  tree.last.child = node;      // Node ist ChildNode
  node.parent     = tree.last; // Node hat den Elternbezug
  tree.first      = node;      // Node ist Listenanfang
  tree.last       = node;      // Node ist Letzter
 }
}
