/*
Jaxe - Editeur XML en Java

Copyright (C) 2008 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformément aux dispositions de la Licence Publique Générale GNU, telle que publiée par la Free Software Foundation ; version 2 de la licence, ou encore (à votre choix) toute version ultérieure.

Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans même la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de détail, voir la Licence Publique Générale GNU .

Vous devez avoir reçu un exemplaire de la Licence Publique Générale GNU en même temps que ce programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/
package jaxe;

import java.awt.Toolkit;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.w3c.dom.Element;

/**
 * The Default ErrorHandler
 * @author tasche
 */
public class ErrorHandler implements ErrorHandlerIf {
    /**
     * Logger for this class
     */
    private static final Logger LOG = Logger.getLogger(ErrorHandler.class);

    private final JaxeDocument doc;

    /**
     * @param document
     */
    public ErrorHandler(final JaxeDocument document) {
        doc = document;
    }

    public void childNotAllowedInParentdef(final Element parentdef, final Element defbalise) {
        final ArrayList<String> autorisees = doc.cfg.nomsSousElements(parentdef);
        final StringBuilder infos = new StringBuilder();
        infos.append(JaxeResourceBundle.getRB().getString("insertion.BalisesAutorisees"));
        infos.append(" ");
        infos.append(doc.cfg.nomElement(parentdef));
        infos.append(":");
        infos.append(Jaxe.newline);
        final StringBuilder infos1 = new StringBuilder();
        int nbnl = 0;
        final Iterator<String> autIt = autorisees.iterator();
        while (autIt.hasNext()) {
            infos1.append(autIt.next());
            if (autIt.hasNext()) {
                infos1.append(", ");
            }
            if (nbnl < infos1.length()/80) {
                infos1.append(Jaxe.newline);
                nbnl++;
            }
        }
        infos.append(infos1);
        if (defbalise != null) {
            infos.append(Jaxe.newline);
            infos.append(Jaxe.newline);
            infos.append(JaxeResourceBundle.getRB().getString("insertion.BalisesParents"));
            infos.append(" ");
            infos.append(doc.cfg.nomElement(defbalise));
            infos.append(": ");
            infos.append(Jaxe.newline);
            final ArrayList<String> lparents = doc.cfg.nomsParents(defbalise);
            final StringBuilder infos2 = new StringBuilder();
            nbnl = 0;
            final Iterator<String> parIt = lparents.iterator();
            while (parIt.hasNext()) {
                infos2.append(parIt.next());
                if (parIt.hasNext()) {
                    infos2.append(", ");
                }
                if (nbnl < infos2.length()/80) {
                    infos2.append(Jaxe.newline);
                    nbnl++;
                }
            }
            infos.append(infos2);
        }
        JOptionPane.showMessageDialog(doc.textPane.jframe, infos.toString(),
            JaxeResourceBundle.getRB().getString("insertion.InsertionBalise"), JOptionPane.ERROR_MESSAGE);
    }

    public void childNotAllowed(final String expr, final JaxeElement parent, final Element defbalise) {
        final StringBuilder infos = new StringBuilder(); 
        infos.append(JaxeResourceBundle.getRB().getString("insertion.Expression"));
        infos.append(" ");
        infos.append(expr);
        
        if (infos.length() > 90) {
            int p=0;
            for (int i=0; i<infos.length(); i++) {
                if (i-p > 80 && (infos.charAt(i) == ' ' || infos.charAt(i) == '|')) {
                    infos.insert(i, "\n");
                    p = i;
                }
            }
        }
        JOptionPane.showMessageDialog(doc.textPane.jframe, infos.toString(),
            JaxeResourceBundle.getRB().getString("insertion.InsertionBalise"), JOptionPane.ERROR_MESSAGE);
    }

    public void notInRootError(final Element defbalise) {
        Toolkit.getDefaultToolkit().beep();
        JOptionPane.showMessageDialog(doc.textPane.jframe,
            JaxeResourceBundle.getRB().getString("insertion.SousRacine"),
            JaxeResourceBundle.getRB().getString("insertion.InsertionBalise"),
            JOptionPane.ERROR_MESSAGE);
    }

    public void editNotAllowed(final JaxeElement parent, final Element defbalise) {
        Toolkit.getDefaultToolkit().beep();
        JOptionPane.showMessageDialog(doc.textPane.jframe,
            JaxeResourceBundle.getRB().getString("insertion.EditionInterdite") +
            " " + parent.noeud.getNodeName(),
            JaxeResourceBundle.getRB().getString("insertion.InsertionBalise"),
            JOptionPane.ERROR_MESSAGE);
    }
        
    public void textNotAllowed(final JaxeElement element) {
        final String infos = JaxeResourceBundle.getRB().getString("erreur.InsertionInterdite") + " " + element.noeud.getNodeName();
        JOptionPane.showMessageDialog(doc.textPane.jframe, infos,
            JaxeResourceBundle.getRB().getString("document.Insertion"), JOptionPane.ERROR_MESSAGE);

    }

}
