package nat.convertisseur;

import gestionnaires.GestionnaireErreur;
import nat.Nat;
import nat.Transcription;
// *** writer2latex ***
/*import writer2latex.Writer2XHTML;
import writer2latex.office.MIMETypes;*/
//import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
//import java.util.Enumeration;
import writer2latex.api.*;
import writer2latex.office.MIMETypes;

/**
 * Convertisseur de documents odt; utilise writer2xhtml pour convertir en xhtml puis {@link ConvertisseurXML}
 * pour convertir au format interne
 * @author bruno
 *
 */
public class ConvertisseurOpenOffice extends ConvertisseurXML
{
	/**
	 * Constructeur
	 * @param source l'adresse du fichier source
	 * @param cible l'adresse du fichier cible
	 */
	public ConvertisseurOpenOffice(String source, String cible){super(source, cible);}
	/**
	 * Redéfinition de {@link Convertisseur#convertir(GestionnaireErreur)}
	 * <p>Convertit d'abord le fichier odt en fichier xhtml avec <code>writer2xhtml</code> (création du fichier temporaire 
	 * {@link Transcription#fTempXHTML}</p>.
	 * <p>Convertit ensuite le fichier {@link Transcription#fTempXHTML} au format interne.
	 */
	public boolean convertir(GestionnaireErreur gest)
	{
		tempsExecution = System.currentTimeMillis();
		boolean retour;
		gest.afficheMessage("** Conversion en XHTML avec Writer2XHTML...",Nat.LOG_VERBEUX);
		//Create a XHTML converter
		Converter converter = ConverterFactory.createConverter(MIMETypes.XHTML_MATHML);

		//Create a configuration
		Config config = converter.getConfig();
		try
		{
			config.read(new FileInputStream("writer2latex/xhtml/config/cleanxhtml.xml"));
			config.setOption("inputencoding","utf-8");
			config.setOption("use_named_entities", "true");
			//Convert the document
			ConverterResult result = converter.convert(new FileInputStream(source),Transcription.fTempXHTML);
			result.write(new File("./"));
			
			/*Enumeration docEnum = result.getMasterDocument().;

			while (docEnum.hasMoreElements()) 
			{
				OutputFile docOut = (OutputFile) docEnum.nextElement();
			    FileOutputStream fos = new FileOutputStream(docOut.getFileName());
			    docOut.write(fos);
			    fos.flush();
			    fos.close();
			}*/
		}
		catch(Exception e)
		{
			gest.afficheMessage("Problème lors de la conversion avec Writer2XHTML",Nat.LOG_SILENCIEUX); e.printStackTrace();
			e.printStackTrace();
		}
		/*Write the files
		Enumeration docEnum = dataOut.getDocumentEnumeration();
		while (docEnum.hasMoreElements())
		{
			OutputFile docOut = (OutputFile) docEnum.nextElement();
		    FileOutputStream fos = new FileOutputStream(docOut.getFileName());
		    docOut.write(fos);
		    fos.flush();
		    fos.close();
		}*/
		//conversion de xhtml vers interne
		ConvertisseurXML convXML = new ConvertisseurXML(Transcription.fTempXHTML, cible);
		retour = convXML.convertir(gest);
		tempsExecution = System.currentTimeMillis() - tempsExecution;
		return retour;
	}
}