package nat.presentateur;

import gestionnaires.GestionnaireErreur;

import java.io.BufferedReader;
import java.io.BufferedWriter;
//import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
//import java.nio.charset.Charset;

import nat.Nat;

//import nat.Nat;
/**
 * Classe abstraite de présentation
 * <p>Les classes <code>Presentateur</code> sont généralement utilisées en fin de traitement,
 * une fois la conversion et la transcription réalisées, afin d'assurer le rendu final du document
 * transcrit</p>
 * <p>Chaque présentateur est tenu d'implémenter la méthode {@link #presenter()} afin de garantir la
 * généricité du code</p>
 */
public abstract class Presentateur
{
	/** L'adresse de la sortie */
	protected String sortie;
	/** le nom de la table braille de présentation */
	protected String tableBraille;
	/** l'adresse du fichier transcrit à présenter*/
	protected String source;
	/** temps d'exécution de la présentation en millisecondes */
	protected long tempsExecution;
	/** une instance de {@link GestionnaireErreur}*/
	protected GestionnaireErreur gest;
	/**
	 * Constructeur
	 * @param g une instance de {@link GestionnaireErreur}
	 * @param src L'adresse de la sortie
	 * @param sor l'adresse du fichier transcrit à présenter
	 * @param tab la table braille de sortie
	 */
	public Presentateur(GestionnaireErreur g,String src, String sor, String tab)
	{
		source= src;
		sortie = sor;
		gest = g;
		tableBraille = tab ;
	}
	/** Méthode d'accès,
	 * @return {@link #tempsExecution}*/
	public long donneTempsExecution(){return tempsExecution;}
	/**
	 * Recopie le fichier source dans le fichier sortie en changeant l'encodage
	 * @param charsetSource encodage du fichier source
	 * @param charsetCible encodage du fichier de sortie
	 */
	protected void encode(String charsetSource, String charsetCible)
	{
		try 
		{
			BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(source),charsetSource));
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(sortie),charsetCible));
			String ligne;
			while ((ligne = br.readLine()) != null)
			{
				bw.write(ligne+"\n");
				//System.err.println(ligne);
			}
			br.close();
			bw.close();
		}
		catch (FileNotFoundException fnfe)
		{
			gest.afficheMessage("Erreur, file not found", Nat.LOG_SILENCIEUX);
			fnfe.printStackTrace();
		}
		catch(IOException ioe)
		{
			gest.afficheMessage("Erreur entrée sortie", Nat.LOG_SILENCIEUX);
			ioe.printStackTrace();
		}
	finally
		{
			//TODO voir le truc pour fermer proprement dans tous les cas
		}
	}
	/**
	 * Méthode de présentation (rendu) de la transcription
	 * @return true si la présentation s'est déroulée correctement, false sinon
	 */
	public abstract boolean presenter();
}