package nat.presentateur;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import nat.ConfigNat;
import nat.Nat;

import org.w3c.dom.Document;

import gestionnaires.GestionnaireErreur;
/**
 * Réalise la présentation du fichier transcrit en appliquant une feuille
 * xsl de transformation, puis en encodant le fichier de sortie dans le
 * charset désiré.
 * @author bruno
 *
 */
public class PresentateurMEP extends PresentateurSans
{
	/** adresse de la feuille xsl de mise en page */
	protected String filtre = "xsl/miseEnPage.xsl";
	/** adresse du fichier temporaire produit après l'application de la feuille xsl de mise en page */
	protected String ftmp = "tmp/tmp-pass2.txt";
	/**
	 * Constructeur
	 * @param gest une instance de {@link GestionnaireErreur}
	 * @param src L'adresse de la sortie
	 * @param sor l'adresse du fichier transcrit à présenter
	 * @param charset encodage du fichier de sortie
	 * @param tab la table braille de sortie
	 */
	public PresentateurMEP(GestionnaireErreur gest, String charset, String src, String sor, String tab)
	{
		super(gest,  charset,src, sor, tab);
		if(!ConfigNat.getCurrentConfig().getMep()){filtre="xsl/no-mep.xsl";}
	}

	/**
	 * Redéfinition de {@link PresentateurSans#presenter()}
	 * <p>Réalise la présentation du fichier transcrit en appliquant une feuille
	 * xsl de transformation, puis en encodant le fichier de sortie dans le
	 * charset désiré.</p>
	 */
	public boolean presenter() 
	{
		long tempsDebut = System.currentTimeMillis();
		gest.afficheMessage("\nDébut de la mise en forme du document ... ok\n",Nat.LOG_SILENCIEUX);
		boolean retour = true;
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		//configuration de la fabrique
		factory.setNamespaceAware(true);
		factory.setValidating(true);
		factory.setIgnoringElementContentWhitespace(true);
		factory.setIgnoringComments(true);
		factory.setIgnoringElementContentWhitespace(false);
		try 
		{
			// sinon, génère parfois des null pointer exp au parsage (problème avec les simples quote)
			factory.setFeature("http://apache.org/xml/features/dom/defer-node-expansion", false);
			DocumentBuilder builder = factory.newDocumentBuilder();
			gest.afficheMessage("** Parsage avec SAX du fichier interne à mettre en page...",Nat.LOG_VERBEUX);
			builder.setErrorHandler(gest);
			gest.afficheMessage("   Entree :"+source+";filtre :"+filtre+";",Nat.LOG_VERBEUX);
			Document doc = builder.parse(new FileInputStream(source),"UTF-8"); //(new File(entree));
			doc.setStrictErrorChecking(true);
			gest.afficheMessage("ok\n** Initialisation et lecture de la feuille de style ...",Nat.LOG_VERBEUX);
			TransformerFactory transformFactory = TransformerFactory.newInstance();
			StreamSource styleSource = new StreamSource(new File(filtre));
			// lire le style
			
			Transformer transform = transformFactory.newTransformer(styleSource);
			//transform.setOutputProperty(name, value)
			// conformer le transformeur au style
			DOMSource in = new DOMSource(doc);
			gest.afficheMessage("ok\n** Création du fichier de sortie ...",Nat.LOG_VERBEUX);
			// Création du fichier de sortie
			File file = new File(ftmp);
			///Result resultat = new StreamResult(fichier);
			StreamResult out = new StreamResult(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file),"UTF-8")));
			gest.afficheMessage("ok\n** Transformation du document ...",Nat.LOG_VERBEUX);
			transform.transform(in, out);
			// transformer selon le style 
			
			//application du traitement de l'encodage (ce que faisait avant PresentateurSans)
			source = ftmp;
			gest.afficheMessage("ok\n** Encodage...",Nat.LOG_VERBEUX);
			encode();
			tempsExecution = System.currentTimeMillis() - tempsDebut;
		}
		catch (Exception e)  
		{
			gest.setException(e);
			gest.gestionErreur();
			retour = false;
		}

		return retour;
	}

}
