package ui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import nat.ConfigNat;
/**
 * Onglet de configuration des options avancées
 * @author bruno
 *
 */
public class ConfAvance extends OngletConf
{
	/** pour la sérialisation (non utilisé)*/
	private static final long serialVersionUID = 1L;
	/** case à cocher activant ou non Saxon plutôt que Xalan
	 * <p>Saxon est normallement l'implémentation xsl à utiliser, car xalan ne fait pas de xsl2</p>
	 */
	private JCheckBox jcbXsltProc = new JCheckBox("Saxon au lieu de Xalan (devrait rester coché dans le cas d'une utilisation normale de NAT)");
	/** JSpinner nombre de fichiers de log */
	JSpinner jsNbFichLog;
	/** JSpinner taille max en Ko du fichier de log */
	JSpinner jsTailleLog;
	/** Constructeur*/
	public ConfAvance()
	{
		super();	
		getAccessibleContext().setAccessibleDescription("Activez cet onglet pour afficher les options avancées");
		getAccessibleContext().setAccessibleName("Onglet contenant les options avancées (experts)");
		
		jcbXsltProc.setSelected(ConfigNat.getCurrentConfig().getSaxonAsXsltProcessor());
		jcbXsltProc.getAccessibleContext().setAccessibleName("Case à cocher utiliser Saxon");
		jcbXsltProc.getAccessibleContext().setAccessibleDescription("Cocher la case pour utiliser saxon");
		jcbXsltProc.setToolTipText("Utiliser le processeur xslt Saxon et pas Xalan (usage normal de NAT) (Alt+x)");
		jcbXsltProc.setMnemonic('x');
		
		jsNbFichLog = new JSpinner(new SpinnerNumberModel(ConfigNat.getCurrentConfig().getNbLogFiles(), 1, 5, 1));
		jsNbFichLog.getAccessibleContext().setAccessibleName("Liste de valeurs pour le nombre de fichiers de log à utiliser");
		jsNbFichLog.getAccessibleContext().setAccessibleDescription("Sélectionnez avec les flèches le nombre de fichiers de log à utiliser");
		jsNbFichLog.setToolTipText("Sélectionnez le nombre de fichiers de log à utiliser (Alt+b)");
		
		JLabel lJsNbFichLog = new JLabel("Nombre de fichiers tournants de log:");
		lJsNbFichLog.setLabelFor(jsNbFichLog);
		lJsNbFichLog.setDisplayedMnemonic('b');
		
		jsTailleLog = new JSpinner(new SpinnerNumberModel(ConfigNat.getCurrentConfig().getLogFileSize(), 1, 3000, 100));
		jsTailleLog.getAccessibleContext().setAccessibleName("Liste de valeurs pour la taille minimale en Ko déclenchant la rotation des logs");
		jsTailleLog.getAccessibleContext().setAccessibleDescription("Sélectionnez avec les flèches la taille minimale déclenchant la rotation des logs");
		jsTailleLog.setToolTipText("Sélectionnez la taille minimale déclenchant la rotation des fichiers de log (Alt+m)");
		
		JLabel lJsTailleLog = new JLabel("Taille minimale pour rotation des logs (en Ko):");
		lJsTailleLog.setLabelFor(jsTailleLog);
		lJsTailleLog.setDisplayedMnemonic('m');
		
		/*********
		 * Mise en page
		 */
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(3,3,3,3);
		GridBagLayout gbl = new GridBagLayout();
		
		//setLayout(gbl);
		
		JPanel p = new JPanel();
		p.setLayout(gbl);
		
		gbc.gridx=0;
		gbc.gridy = 0;
		gbc.gridwidth = 2;
		p.add(jcbXsltProc,gbc);
		
		gbc.gridy++;
		gbc.gridwidth = 1;
		p.add(lJsNbFichLog,gbc);
		
		gbc.gridx++;
		p.add(jsNbFichLog,gbc);
		
		gbc.gridy++;
		gbc.gridx=0;
		p.add(lJsTailleLog,gbc);
		
		gbc.gridx++;
		p.add(jsTailleLog,gbc);
		
		add(p);
	}

	/**
	 * enregistre les options
	 * @see ui.SavableTabbedConfigurationPane#enregistrer(java.lang.String)
	 */
	public boolean enregistrer(String f)
	{
		ConfigNat.getCurrentConfig().setFichierConf(f);
		return enregistrer();
	}
	/**
	 * enregistre les options
	 * @see ui.SavableTabbedConfigurationPane#enregistrer()
	 */
	public boolean enregistrer()
	{
		boolean retour = true;
		ConfigNat.getCurrentConfig().setSaxonAsXsltProcessor(jcbXsltProc.isSelected());
		ConfigNat.getCurrentConfig().setLogFileSize(((Integer)jsTailleLog.getValue()).intValue());
		ConfigNat.getCurrentConfig().setNbLogFiles(((Integer)jsNbFichLog.getValue()).intValue());
		return retour;
	}
}
