/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret, Frédérick Schwebel, Vivien Guillet
 * Contact: natbraille@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package ui;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import nat.ConfigNat;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;


/**
 * Onglet de configuration de la mise en page avancée
 * @see OngletConf
 * @since 2.0
 * @author bruno
 *
 */
public class ConfMiseEnPageAvancee extends OngletConf implements ChangeListener 
{
	/** pour la sérialisation (non utilisé) */
	private static final long serialVersionUID = 1L;
	
	/** ArrayList des JSpinner pour les titres*/
	private ArrayList<JSpinner> jsTitres = new ArrayList<JSpinner>();
	/** ArrayList pour les JLabel de jsTitres
	 * @see ConfMiseEnPageAvancee#jsTitres
	 */
	private ArrayList<JLabel> jlTitres = new ArrayList<JLabel>();
	/** Variable indiquant s'il y a eu des changements sur les titres */
	private boolean modifTitre = false;
	/** Constante représentannt les niveaux de titres stricts à appliquer */
	private static final String TITRES_STRICTS = "1,2,3,4,5,5,5,5,5";
	/** La table pour l'ajout des caractères de formatage spécifiques*/
	private JTable jtableAjout;
	/** Tableau pour les données de la table */
	private Object[][] donnees;
	
	/**
	 * Constructeur de l'onglet Mise en page avancée
	 *
	 */
	public ConfMiseEnPageAvancee()
	{
		super();
		getAccessibleContext().setAccessibleDescription("Activez cet onglet pour afficher les options de mise en page avancée");
		getAccessibleContext().setAccessibleName("Onglet contenant les options de mise en page avancée");

		/**********
		 * Préparation des composants
		 */
		String[] listeTitre = ConfigNat.getCurrentConfig().getNiveauxTitres().split(",");
		int[] listeT = new int[listeTitre.length];
		
		for(int i=0; i<listeTitre.length;i++){listeT[i]=Integer.parseInt(listeTitre[i]);}
		
		for(int i=0;i<listeT.length;i++)
		{
			jlTitres.add(new JLabel("Un titre de niveau "+ (i+1) + " en noir donne un titre braille de niveau "));
			jlTitres.get(i).setDisplayedMnemonic(49 + i);//48 + 1 + i représente le code ascci du char représentant i
			jsTitres.add(new JSpinner(new SpinnerNumberModel(listeT[i], 1, 5, 1)));
			jlTitres.get(i).setLabelFor(jsTitres.get(i));
			jsTitres.get(i).getAccessibleContext().setAccessibleName("Liste déroulante niveau de titre braille ("+(i+1)+")");
			jsTitres.get(i).getAccessibleContext().setAccessibleDescription("Sélectionner avec les flèches le niveau de titre braille ("+(i+1)+")");
			jsTitres.get(i).setToolTipText("niveau du titre braille ("+(i+1)+")");
			jsTitres.get(i).addChangeListener(this);
		}
	    
		/*
		 * Fabrication de la table ajout
		 */
		donnees = chargeTableAjout();
		TableModel tm = new TableModeleAjout(donnees);
		
		jtableAjout = new JTable(tm);
		jtableAjout.getAccessibleContext().setAccessibleName("Table ajout de caractères");
		jtableAjout.getAccessibleContext().setAccessibleDescription("Table contenant 5 colonnes: avant, compter le rajout dans la taille de la ligne, " +
			"élément considéré, après, compter le rajout après dans la longueur de la ligne");
		jtableAjout.setToolTipText("Entrez les chaines à ajouter (Alt+a)");

		//table.setAutoCreateRowSorter(true);
		TableColumnModel modelesColonnes = jtableAjout.getColumnModel();
        TableColumn modelColonne = (TableColumn) modelesColonnes.getColumn(0);
        modelColonne.setMaxWidth(100);
        modelColonne.setMinWidth(60);
        
        modelColonne = (TableColumn) modelesColonnes.getColumn(1);
        modelColonne.setMaxWidth(60);
        modelColonne.setMinWidth(60);
        
        modelColonne = (TableColumn) modelesColonnes.getColumn(2);
        modelColonne.setMaxWidth(150);
        modelColonne.setMinWidth(100);
        
        modelColonne = (TableColumn) modelesColonnes.getColumn(3);
        modelColonne.setMaxWidth(100);
        modelColonne.setMinWidth(60);
        
        modelColonne = (TableColumn) modelesColonnes.getColumn(4);
        modelColonne.setMaxWidth(60);
        modelColonne.setMinWidth(60);
        
		/*
		 * Mise en page
		 */
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.insets = new Insets(3,3,3,3);
		GridBagLayout gbl = new GridBagLayout();
		JPanel panTr = new JPanel();
		panTr.setLayout(gbl);
		
		JLabel titre = new JLabel("<html><h3>Paramétrage avancé de la mise en page</h3></html>");	
		
		gbc.anchor = GridBagConstraints.WEST;
		
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth=3;
		gbl.setConstraints(titre, gbc);
		panTr.add(titre);
		
		JLabel titre2 = new JLabel("<html><h4>Titres</h4></html>");	
		gbc.gridwidth = 1;
		gbc.gridy++;
		gbl.setConstraints(titre2, gbc);
		panTr.add(titre2);
		
		for(int i=0;i<jsTitres.size();i++)
		{
			gbc.gridy++;
			gbc.gridx=1;
			gbc.gridwidth=2;
			gbl.setConstraints(jlTitres.get(i), gbc);
			panTr.add(jlTitres.get(i));
			
			gbc.gridx=gbc.gridx+2;
			gbc.gridwidth=1;
			gbl.setConstraints(jsTitres.get(i), gbc);
			panTr.add(jsTitres.get(i));
		}
		/*
		 * Table d'ajout des caractères
		 */
		JLabel titre3 = new JLabel("<html><h4>Chaines à <u>a</u>jouter</h4></html>");
		titre3.setLabelFor(jtableAjout);
		titre3.setDisplayedMnemonic('a');
		gbc.gridwidth = 4;
		gbc.gridx=0;
		gbc.gridy++;
		gbl.setConstraints(titre3, gbc);
		panTr.add(titre3);
		
		gbc.gridy++;
		gbc.insets = new Insets(0,0,0,0);
		gbc.gridx=1;
		gbl.setConstraints(jtableAjout.getTableHeader(), gbc);
		panTr.add(jtableAjout.getTableHeader());
		
		gbc.gridy++;
		gbl.setConstraints(jtableAjout, gbc);
		panTr.add(jtableAjout);
		
		// TODO à virer
		JLabel vireMoi = new JLabel("<html><font color=\"red\">Les colonnes \"Compter\" ne servent pas pour l'instant</font></html>");
		gbc.gridy++;
		gbl.setConstraints(vireMoi, gbc);
		panTr.add(vireMoi);
		
		setLayout(new BorderLayout());
		add(panTr, BorderLayout.NORTH);

		//bug de sun pour la maj de la table
		jtableAjout.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
	}	
	


  

	/**
	 *  Initialise la table pour les ajouts
	 * @return la double table contenant les rajouts
	 * @see ConfMiseEnPageAvancee#jtableAjout
	 */
	private Object[][] chargeTableAjout()
	{
		String [] listeAjout = {"document","paragraphe","ligne","mathématique","littéraire","musique"};
		Object[][] retour = new Object[listeAjout.length][5];
		//initialisation
		String raj = ConfigNat.getCurrentConfig().getRajout();
		//		String[] rajouts = raj.split(",");

		String[] rajouts = ConfigNat.intelliSplit(raj,",");

		String cpt = ConfigNat.getCurrentConfig().getRajoutCompte();
		String [] comptes = cpt.split(",");
		for(int i=0;i<listeAjout.length;i++)
		{
			retour[i][0] = rajouts[2*i].substring(1,rajouts[2*i].length()-1 );
			retour[i][1] = new Boolean(comptes[2*i]);
			retour[i][2] = listeAjout[i];
			retour[i][3] = rajouts[2*i+1].substring(1,rajouts[2*i+1].length()-1 );
			retour[i][4] = new Boolean(comptes[2*i+1]);
		}
		
		return retour;
	}



	/**
	 * Enregistre les options de l'onglet
	 * @see ui.SavableTabbedConfigurationPane#enregistrer(java.lang.String)
	 */
	public boolean enregistrer(String f)
	{
		ConfigNat.getCurrentConfig().setFichierConf(f);
		return enregistrer();
	}
	/**
	 * Enregistre les options de l'onglet
	 * @see ui.SavableTabbedConfigurationPane#enregistrer()
	 */
	public boolean enregistrer()
	{
		boolean retour = true;
		/*
		 * les titres
		 */ 
		if(ConfigNat.getCurrentConfig().getTitresStricts())
		{
			ConfigNat.getCurrentConfig().setNiveauxTitres(TITRES_STRICTS);
		}
		else
		{
			boolean saisieOK = true;
			//premier niveau
			int mini = ((Integer)(jsTitres.get(0).getValue())).intValue();
			String titres="" + mini;
			//niveaux suivants
			for(int i=1; i<jsTitres.size();i++)
			{
				int valeur = ((Integer)(jsTitres.get(i).getValue())).intValue();
				titres = titres + "," + valeur;
				saisieOK = saisieOK && (mini<=valeur);
				mini=valeur;
			}
			int rep=JOptionPane.YES_OPTION;
			if(!saisieOK && modifTitre)//la saisie n'est pas bonne ET il y a eu des changements
			{
				rep = JOptionPane.showConfirmDialog(
					    this,
					    "Les niveaux de titres en braille ne sont pas ordonnés comme les niveaux de titre en noir. Voulez-vous continuer?",
					    "Avertissement",
					    JOptionPane.YES_NO_OPTION);
			}
			if(rep==JOptionPane.YES_OPTION)
			{
				ConfigNat.getCurrentConfig().setNiveauxTitres(titres);
			}
		}
		/*
		 * Les rajouts
		 */
		String ajouts ="";
		String comptes="";
		for(int i=0;i<donnees.length;i++)
		{
		    String before = (String) donnees[i][0];
		    String after  = (String) donnees[i][3];
		    before = before.replaceAll(",",",,");
		    after = after.replaceAll(",",",,");	   
		    ajouts = ajouts+"'"+before+"','"+after+"',";
		    comptes= comptes + donnees[i][1]+","+donnees[i][4]+",";
		}
		//Suppression du dernier séparateur
		ajouts = ajouts.substring(0,ajouts.length()-1);
		comptes = comptes.substring(0,comptes.length()-1);
		
		ConfigNat.getCurrentConfig().setRajout(ajouts);
		ConfigNat.getCurrentConfig().setRajoutCompte(comptes);
		
		ConfigNat.getCurrentConfig().sauvegarder();
		return retour;
	}
	/**
	 * Implémentation de ChangeListener
	 * mets modifTitre à vrai pour indiquer qu'il y a eu un changement dans la configuration des titres
	 * @see ConfMiseEnPageAvancee#modifTitre
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	public void stateChanged(ChangeEvent arg0) {modifTitre = true;}
	/**
	 * Classe interne de modèle de table pour la table d'ajout
	 * @see ConfMiseEnPageAvancee#jtableAjout
	 * @author bruno
	 *
	 */
	private class TableModeleAjout extends AbstractTableModel
	{
		/** Pour la sérialisation (non utilisé) */
		private static final long serialVersionUID = 1L;
		/** La matrice des objets du tableau */
		private Object[][] data;
		/** le tableau des classes associées à chaque colonne */
		private Class<?>[] colClass = new Class<?>[]{String.class,Boolean.class,String.class,String.class,Boolean.class};
		/** le tableau des noms des colonnes */
		private String[] columnNames = new String[]{"avant", "compter", "élément","après","compter"};
		/* le tableau des toolType text pour les noms des colonnes /
		private String[] columnToolTipText = new String[]{"chaine à insérer avant l'élément",
				"prendre en compte la longueur de la chaine insérée avant l'élément dans la longueur de la ligne",
				"type d'élément concerné",
				"chaine à insérer après l'élément",
				"prendre en compte la longueur de la chaine insérée après l'élément dans la longueur de la ligne"};
		*/
		/** 
		 * Constructeur
		 * @param o Les objets du tableau
		 */
		public TableModeleAjout(Object[][] o)
		{
			super();
			data=o;
		}
		
		/**
		 * Renvoie le nom de la colonne n° col
		 * @param col Le numéro de la colonne
		 * @return le nom de la colonne
		 */
		public String getColumnName(int col) {return columnNames[col].toString();}
		/**
		 * Modifie la valeur de l'objet de la table situé en (row,col)
		 * redéfinition;
		 * @param value la valeur de modification
		 * @param row la ligne
		 * @param col la colonne de l'objet 
		 */
		public void setValueAt(Object value, int row, int col) {
	        data[row][col] = value;
	        fireTableCellUpdated(row, col);
	    }

		/**
		 * Implémentation: Donne le nombre de colonnes de la table
		 * @return le nombre de colonnes
		 */
		public int getColumnCount() {return data[0].length;}
		/**
		 * Implémentation: Donne le nombre de lignes de la table
		 * @return le nombre de lignes
		 */
		public int getRowCount() {return data.length;}
		/**
		 * Implémentation: renvoie l'objet situé en (row, col)
		 * @param row la ligne de l'objet
		 * @param col la colonne de l'objet 
		 * @return l'objet
		 */
		public Object getValueAt(int row, int col) {return data[row][col];}
		/**
		 * Implémentation: renvoie vrai si l'objet situé en (row, col) est éditable
		 * @param row la ligne de l'objet
		 * @param col la colonne de l'objet 
		 * @return vrai si l'objet situé en (row, col) est éditable
		 */
		public boolean isCellEditable(int row, int col){return true;}
		/**
		 * Redéfinition: renvoie la class des objets de la colonne col
		 * @param col la colonne
		 * @return la classe des objet de la colonne
		 */
		public Class<?> getColumnClass(int col) {return colClass[col];}
	}
}
