/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package ui;

import gestionnaires.GestionnaireErreur;
import gestionnaires.GestionnaireMajTabBraille;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JButton;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.ArrayList;
import java.io.IOException;
//import javax.swing.ImageIcon;

import nat.ConfigNat;
import nat.Nat;
/**
 * Fenêtre graphique de configuration des options de NAT
 * @author bruno
 *
 */
public class Configuration extends JFrame implements ActionListener, WindowListener
{
    /** Pour la sérialisation (non utilisé) */
    private static final long serialVersionUID = 1L;
    /** JTabbedPane contenant les onglets de configuration */
    private JTabbedPane onglets = new JTabbedPane();
    /** Onglet de configuration générale */
    private ConfGeneral panGen;
    /** liste de tous les onglets de configuration */
    private ArrayList<OngletConf> listOnglets = new ArrayList<OngletConf>();
    /** JLabel indiquant la configuration éditée */
    private JLabel lTitreConfig = new JLabel ("<html><h3>Edition de la configuration "
					      + ((ConfigNat.getCurrentConfig().getIsSysConfig())?"(système)":"") 
					      + " : "
					      + ConfigNat.getCurrentConfig().getName() 										
					      + "</h3></html>");
    //private JLabel lInfosConfig  = new JLabel ("<b>"+ConfigNat.getCurrentConfig().getInfos() +"</b>");											
    /** JPanel du titre */
    private JPanel panelTitre = new JPanel();
    /** JPanel conteannt les boutons d'enregistrement */
    private JPanel lesBoutons = new JPanel();
    /** Annule les modifications */
    private JButton btAnnuler = new JButton("Annuler",new ImageIcon("ui/icon/edit-undo.png"));
    /** Enregistre les modifications dans la configuration courante */
    private JButton btEnregistrer = new JButton("Enregistrer",new ImageIcon("ui/icon/document-save.png"));
    /** Enregistre les options dans une nouvelle configuration */
    private JButton btEnregistrerSous = new JButton("Enregistrer sous...",new ImageIcon("ui/icon/document-save-as.png"));
    /** Instance de la fenêtre principale de NAT qui a construit l'instance de Configuration */
    private FenetrePrinc fPrinc;
    /** une instance de {@link gestionnaires.GestionnaireMajTabBraille}*/
    private GestionnaireMajTabBraille gmtb;
    /**
     * Constructeur
     * @param fp la fenêtre principale appelant le constructeur
     */
    public Configuration(FenetrePrinc fp)
    {
		super("Configuration de NAT");
		fPrinc = fp;
		addWindowListener(this);
		ConfEmbossage panConfEmbossage = new ConfEmbossage();
		panGen = new ConfGeneral(this);
			
		listOnglets.add(panConfEmbossage);
		listOnglets.add(new ConfConversion());
		listOnglets.add(new ConfTranscription());
		listOnglets.add(new ConfMiseEnPage());
		listOnglets.add(new ConfMiseEnPageAvancee());
		listOnglets.add(new ConfPostTraitement());//,this);
		listOnglets.add(new ConfInterface());
		listOnglets.add(new ConfAvance());
		listOnglets.add(panGen);
			
		gmtb = new GestionnaireMajTabBraille (panGen.getComboTables(), panConfEmbossage.getComboTables());
		onglets.addTab("Général",new ImageIcon("ui/icon/document-properties.png"),panGen,"Regroupe les options de configuration les plus génériques (Alt+g)");
		onglets.setMnemonicAt(0, KeyEvent.VK_G);
		onglets.addTab("Transcription",new ImageIcon("ui/icon/stock_script.png"),listOnglets.get(2),"Regroupe les options pour un paramétrage fin de la transcription");
		onglets.setMnemonicAt(1, '2');
		onglets.addTab("Mise en page", new ImageIcon("ui/icon/document-page-setup.png"),listOnglets.get(3),"Regroupe les options de mise en page");
		onglets.setMnemonicAt(2, '3');
		onglets.addTab("MEP avancée", new ImageIcon("ui/icon/document-page-setup-advanced.png"),listOnglets.get(4),"Regroupe les options de mise en page avancée");
		onglets.setMnemonicAt(3, '4');
		onglets.addTab("Embossage",new ImageIcon("ui/icon/stock_print-setup.png"),panConfEmbossage,"Regroupe les options pour l'embossage");
		onglets.setMnemonicAt(4, '5');
		onglets.addTab("Interface",new ImageIcon("ui/icon/gnome-settings-ui-behavior.png"),listOnglets.get(6),"Regroupe les options de configuration de l'interface graphique");
		onglets.setMnemonicAt(5, '6');
		if(!(ConfigNat.getCurrentConfig().getNiveauLog() < Nat.LOG_DEBUG))
		    {
			onglets.addTab("Avancé",new ImageIcon("ui/icon/stock_form-properties.png"),listOnglets.get(7),"Regroupe les réglages techniques avancés de NAT");
			onglets.setMnemonicAt(6, '7');
		    }
		//onglets.addTab("Conversion",new ImageIcon("ui/icon/stock_filter-data-by-criteria.png"),listOnglets.get(1),"Regroupe les options des convertisseurs");
	
		btAnnuler.addActionListener(this);
		btAnnuler.getAccessibleContext().setAccessibleName("Bouton annuler");
		btAnnuler.getAccessibleContext().setAccessibleDescription("Valider pour annuler les modifications et fermer la fenêtre");
		btAnnuler.setToolTipText("Annuler et fermer la fenêtre (Alt+z) ou (Alt+F4)");
		btAnnuler.setMnemonic('z');
			
		if ((!(ConfigNat.getCurrentConfig().getIsSysConfig())) && (ConfigNat.getCurrentConfig().getFichierConf() != null))
		    {
		        btEnregistrer.addActionListener(this);
		    } 
		else {btEnregistrer.setEnabled(false);}
	
		btEnregistrer.getAccessibleContext().setAccessibleName("Bouton enregistrer");
		btEnregistrer.getAccessibleContext().setAccessibleDescription("Valider pour enregister les modifications et fermer la fenêtre");
		btEnregistrer.setToolTipText("Enregistrer et fermer la fenêtre (Alt+s)");
		btEnregistrer.setMnemonic('s');
			
		btEnregistrerSous.addActionListener(this);
		btEnregistrerSous.getAccessibleContext().setAccessibleName("Bouton enregistrer sous");
		btEnregistrerSous.getAccessibleContext().setAccessibleDescription("Valider pour enregistrer la configuration sous un nouveau nom");
		btEnregistrerSous.setToolTipText("Enregistrer la configuration sous un nouveau nom (Alt+n)");
		btEnregistrerSous.setMnemonic('n');
	
		lesBoutons.add(btEnregistrer);
		lesBoutons.add(btEnregistrerSous);
		lesBoutons.add(btAnnuler);
	
		panelTitre.add(lTitreConfig);
		add("North", panelTitre);
		add("Center",onglets);
		add("South",lesBoutons);
		pack();
		if(ConfigNat.getCurrentConfig().getCentrerFenetre())
		    {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension size = this.getPreferredSize();
			screenSize.height = screenSize.height/2;
			screenSize.width = screenSize.width/2;
			size.height = size.height/2;
			size.width = size.width/2;
			int y = screenSize.height - size.height;
			int x = screenSize.width - size.width;
			setLocation(x, y);
		    }
	
	    }
	    /** Méthode d'accès en lecture à {@link #gmtb}
	     * @return {@link #gmtb} 
	     **/
	    public GestionnaireMajTabBraille getGmtb (){return gmtb ;}
	    /**
	     * ferme la fenêtre et indique à {@link #fPrinc} que la fenêtre est fermée
	     * @see java.awt.Window#dispose()
	     */
	    public void dispose()
	    {
		super.dispose();
		fPrinc.setOptionsOuvertes(false);
    }
    /**
     * Gère les actions sur les boutons d'enregistrement
     * @see #btAnnuler
     * @see #btEnregistrer
     * @see #btEnregistrerSous
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent evt)
    {
	if (evt.getSource() == btAnnuler)
	    {
		this.dispose();
		try
		    {
			this.finalize();
		    }
		catch (Throwable e){e.printStackTrace();}
	    }
	else if (evt.getSource() == btEnregistrer)
	    {
		if (saveAll())
		    {
			//String nomConf = ConfigNat.getCurrentConfig().getShortFichierConf();
			ConfigNat.getCurrentConfig().saveFilterConf(ConfigNat.getCurrentConfig().getFichierConf());
			fPrinc.chargeConfigurations();
			this.dispose();
			try {this.finalize();}
			catch (Throwable e){e.printStackTrace();}
		    }
	    }
	else if(evt.getSource()==btEnregistrerSous)
	    {	
		String nomConf;
		if (ConfigNat.getCurrentConfig().getFichierConf() != null )
		    {
		        nomConf = ConfigNat.getCurrentConfig().getShortFichierConf();
		    }
		else {nomConf = "defaut";}

		//System.out.println("Enregistrere sous / ancienne : "+nomConf);
		//on demande le nom de la nouvelle config
		String nomNewConf = (String)JOptionPane.showInputDialog(
									this,
									"Nom de la nouvelle configuration :",
									"Conf. actuelle : "+nomConf,
									JOptionPane.PLAIN_MESSAGE,null,null,
									nomConf);
			
		//on vérifie qu'elle n'existe pas déjà
		int reponse = JOptionPane.YES_OPTION ;
		if ((new File(ConfigNat.getUserConfigFolder()+nomNewConf).exists()) && (nomNewConf != null))
		    {
			reponse = JOptionPane.showConfirmDialog(
								this,
								"La configuration\n"+nomNewConf+"\nexiste déjà,\nvoulez-vous l'écraser ?",
								"Confirmation",
								JOptionPane.YES_NO_OPTION);
		    }

		if ((nomNewConf != null) && (nomNewConf.length() > 0) && (reponse == JOptionPane.YES_OPTION))
		    {    
			try
			    {
				String cpn = (new File(ConfigNat.getUserConfigFilterFolder() + nomNewConf).getCanonicalPath());				
				ConfigNat.getCurrentConfig().setFichierConf(cpn);
    			    
				if (saveAll())
				    {
					ConfigNat.getCurrentConfig().setIsSysConfig(false);
					ConfigNat.getCurrentConfig().setName(nomNewConf);
					ConfigNat.getCurrentConfig().saveFilterConf(ConfigNat.getCurrentConfig().getFichierConf());
					//				ConfigNat.getCurrentConfig().sauvegarder();

					fPrinc.chargeConfigurations();//ajout de la nouvelle conf
					this.dispose();
					try{this.finalize();}
					catch (Throwable e){e.printStackTrace();}
				    }
			    }
			catch (IOException ioe)
			    {
				JOptionPane.showMessageDialog(this,
							      "Erreur lors de l'enregistrement\n La configuration na pas pu être sauvegardée",
							      "Erreur",JOptionPane.ERROR_MESSAGE);
			    }
		    }
	    }
    }
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
     */
    public void windowActivated(WindowEvent arg0) {}
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
     */
    public void windowClosed(WindowEvent arg0) {}
    /** 
     * Indique à {@link #fPrinc} que la fenêtre des options est fermée
     * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
     */
    public void windowClosing(WindowEvent arg0) {fPrinc.setOptionsOuvertes(false);}
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
     */
    public void windowDeactivated(WindowEvent arg0) {}
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
     */
    public void windowDeiconified(WindowEvent arg0) {}
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
     */
    public void windowIconified(WindowEvent arg0) {}
    /** ne fait rien
     * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
     */
    public void windowOpened(WindowEvent arg0) {}
    /**
     * Sauvegarde tous les onglets de configuration contenus dans {@link #listOnglets}
     * @return true si les sauvegardes se sont bien déroulées, false sinon
     */
    private boolean saveAll()
    {
	boolean retour = true;
	int i= 0;
	while(i<listOnglets.size()&&retour)
	    {
		retour = listOnglets.get(i).enregistrer();
		i++;
	    }
	return retour;
    }
    /** 
     * Renvoie le gestionnaire d'erreur utilisé dans la fenêtre principale 
     * @return le gestionnaire d'erreur utilisé dans la fenêtre principale
     * */
	public GestionnaireErreur getGestErreur() {return fPrinc.getGestErreur();}
}
