/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package ui;

import formula.MathPanel;
import formula.display.DispObject;
import formula.display.DispString;
import formula.handlers.MathDocHandler;
import gestionnaires.GestionnaireExporter;

import java.awt.BorderLayout;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.io.IOException;

import javax.swing.ImageIcon;
import javax.swing.InputMap;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JCheckBox;
import javax.swing.KeyStroke;
import javax.swing.border.LineBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.undo.UndoManager;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import nat.ConfigNat;
import nat.Nat;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.PrintWriter;
import java.io.RandomAccessFile;
import java.io.StringWriter;

import outils.Embosseur;
import outils.FileToolKit;
import outils.TextSender;

import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Hashtable;

/**
 * Cette classe de l'interface graphique permet d'afficher le fichier transcrit et d'intéragir avec lui
 * (édition, mode perkins, etc.).
 * @author Bruno Mascret
 */
public class EditeurTan extends JFrame implements ActionListener, KeyListener, TextSender, DocumentListener, UndoableEditListener,FocusListener
{
	/** identifiant par défaut pour la sérialisation (non utilisé dans NAT)*/ 
	private static final long serialVersionUID = 1L;
	/** le label contenant le nom du fichier transcrit */
	private JLabel lFichier = new JLabel("Fichier: ");
	/** le panneau central contenant les éléments à afficher et le scrollPane*/
	private JPanel panneauAffichage = new JPanel();
	/** la zone d'affichage principale du fichier */
	private JTextPane resultat = new JTextPane();
	/** la zone d'affichage d'aperçu en noir du fichier */
	private JTextPane apercu = new JTextPane();
	/** le ScrollPane associé au JTextPane resultat */
	private JScrollPane scrollRes;
	/** le ScrollPane associé au JTextPane apercu */
	private JScrollPane scrollAp;
	/** Le panneau contenant les boutons d'action */
	private JPanel lesBoutons = new JPanel();
	/** JButton pour enregistrer le fichier */
	private JButton btEnregistrer = new JButton("Enregistrer",new ImageIcon("ui/icon/document-save.png"));
	/** JButton pour enregistrer le fichier */
	private JButton btEnregistrersous = new JButton("Enregistrer sous",new ImageIcon("ui/icon/document-save-as.png"));
    /** JButton pour fermer la fenêtre */
    private JButton btFermer = new JButton("Fermer",new ImageIcon("ui/icon/exit.png"));
	/** L'adresse du fichier transcrit */
	private String fichier = null;
	/** encodage du fichier transcrit */
	private String encodage ="UTF-8";
	/** Jlabel d'information pour l'enregistrement */
	private JLabel message= new JLabel("");
	/** tableau destiné à recevoir les codes perkins */
	private boolean [] tabPoint = new boolean[6];
	/** JCheckBoxpour l'activation du mode perkins */
	private JCheckBox jcbPerkins = new JCheckBox("Saisie Perkins");
	/** JButton pour lancer l'embossage */
	private JButton btEmbosser = new JButton("Embosser",new ImageIcon("ui/icon/document-print.png"));
	/** JButton aperçu en noir du texte braille saisi*/
	private JButton btApercu = new JButton("Aperçu en noir", new ImageIcon("ui/icon/view-refresh.png"));
	/** JButton aperçu dans un navigateur */
	private JButton btNavigateur = new JButton("<html>Aperçu dans un<br>navigateur <u>w</u>eb</html>", new ImageIcon("ui/icon/web-browser.png"));
	
	
	/** nombre de touches pressées lors d'une saisie perkins */
	private int nbTouches = 0;
	
	/** Objet embosseur, pour l'embossage bien entendu...*/
	private Embosseur embosseur;
	/** table braille à utiliser pour la saisie perkins */
	private String tableBraille = "Brltab";
	/** HashTable pour la correspondance entre les caractères braille saisis en mode perkins et leur
	 * représentation dans la table braille */
	private Hashtable<String,String> ptPerkins = new Hashtable<String,String>();
	
	//fonctionalités pour l'édition
	/** Undo manager */
	private UndoManager undoMng = new UndoManager();
	/** JButton pour annuler frappe */
	private JButton btUndo = new JButton(new ImageIcon("ui/icon/edit-undo.png"));
	/** JButton pour répéter frappe */
	private JButton btRedo = new JButton(new ImageIcon("ui/icon/edit-redo.png"));
	/** Position du curseur **/
	private int positionCurseur = 0;
	/** instance de nat pour lancer les détranscriptions*/
	private Nat nat;
	
	/** 
	 * Construit un objet Editeur
	 * @param emb l'objet Embosseur à utiliser pour l'embossage
	 * @param f adresse du fichier édité
	 * @param nat instance de nat pour lancer les détranscriptions
	 */
	public EditeurTan(Embosseur emb, String f,Nat nat)
	{
	    super("Editeur");
		embosseur = emb;
		fichier = f;
		this.nat=nat;

		setSize(600,400);
		/* 
		 * Boutons 
		 */
		btEnregistrer.addActionListener(this);
		btEnregistrer.getAccessibleContext().setAccessibleName("Bouton enregistrer");
		btEnregistrer.getAccessibleContext().setAccessibleDescription("Valider pour enregistrer le fichier");
		btEnregistrer.setToolTipText("Enregistrer le fichier (Alt+s)");
		btEnregistrer.setMnemonic('s');
		
		btEnregistrersous.addActionListener(new GestionnaireExporter(this,this,GestionnaireExporter.EXPORTER_BRF));
		btEnregistrersous.getAccessibleContext().setAccessibleName("Bouton enregistrer sous");
		btEnregistrersous.getAccessibleContext().setAccessibleDescription("Valider pour enregistrer le fichier avec un nouveau nom");
		btEnregistrersous.setToolTipText("Enregistrer le fichier avec un nouveau nom (Alt+n)");
		btEnregistrersous.setMnemonic('n');
		
		btFermer.addActionListener(this);
		btFermer.getAccessibleContext().setAccessibleName("Bouton fermer l'éditeur");
		btFermer.getAccessibleContext().setAccessibleDescription("Valider pour fermer l'éditeur");
		btFermer.setToolTipText("Fermer l'éditeur (Alt+f)");
		btFermer.setMnemonic('f');
		
		btEmbosser.addActionListener(this);
		btEmbosser.getAccessibleContext().setAccessibleName("Bouton embosser");
		btEmbosser.getAccessibleContext().setAccessibleDescription("Valider pour embosser le document");
		btEmbosser.setToolTipText("Embosser le document (Alt+e)");
		btEmbosser.setMnemonic('e');
		if(embosseur==null){btEmbosser.setEnabled(false);}
		
		btNavigateur.addActionListener(this);
		btNavigateur.getAccessibleContext().setAccessibleName("Bouton Aperçu dans un navigateur web");
		btNavigateur.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir la sortie noire dans le navigateur par défaut");
		btNavigateur.setToolTipText("Aperçu dans le navigateur web par défaut du système (Alt+w)");
		btNavigateur.setMnemonic('w');
		btNavigateur.setEnabled(false);
		
		jcbPerkins.getAccessibleContext().setAccessibleName("Case à cocher saisie Perkins");
		jcbPerkins.getAccessibleContext().setAccessibleDescription("Cocher cette case pour activer ou désactiver la saisie Perkins");
		jcbPerkins.setToolTipText("Saisie Perkins : saisissez les points braille avec FDS et JKL (Alt+p)");
		jcbPerkins.setMnemonic('p');
		jcbPerkins.addActionListener(this);
		
		btUndo.addActionListener(this);
		btUndo.getAccessibleContext().setAccessibleName("Bouton annuler édition");
		btUndo.getAccessibleContext().setAccessibleDescription("Valider pour annuler la dernière modification");
		btUndo.setToolTipText("Annuler la dernière modification (Alt+u)");
		btUndo.setMnemonic('u');
		btUndo.setEnabled(false);
		
		btRedo.addActionListener(this);
		btRedo.getAccessibleContext().setAccessibleName("Bouton rétablir édition");
		btRedo.getAccessibleContext().setAccessibleDescription("Valider pour rétablir la dernière modification");
		btRedo.setToolTipText("Rétablir la dernière modification (Alt+r)");
		btRedo.setMnemonic('r');
		btRedo.setEnabled(false);
		
		btApercu.addActionListener(this);
		btApercu.getAccessibleContext().setAccessibleName("Bouton aperçu en noir");
		btApercu.getAccessibleContext().setAccessibleDescription("Valider pour raffraichir l'aperçu en noir");
		btApercu.setToolTipText("Raffraichir le rendu en noir du texte saisi (Alt+a)");
		btApercu.setMnemonic('a');
		
		scrollRes = new JScrollPane (resultat);
		scrollRes.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollRes.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		scrollAp = new JScrollPane (apercu);
		scrollAp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollAp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,scrollRes,scrollAp);
		splitPane.setOneTouchExpandable(true);
		//splitPane.setDividerLocation(scrollAp.getHeight());
		
		//Provide minimum sizes for the two components in the split pane
		Dimension minimumSize = new Dimension(100, 50);
		scrollRes.setMinimumSize(minimumSize);
		scrollAp.setMinimumSize(minimumSize);

		
		/*
		 * Mise en page
		 */
		lesBoutons.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();

		
		c.gridx=0;
		c.gridy=0;
		c.gridwidth = 3;
		lesBoutons.add(message,c);
		c.gridwidth=1;
		c.gridy++;
		lesBoutons.add(btEnregistrer,c);
		c.gridx++;
		lesBoutons.add(btEnregistrersous,c);
		c.gridx++;
		lesBoutons.add(btFermer,c);
		
		panneauAffichage.setLayout(new BorderLayout());
		panneauAffichage.add("Center",splitPane);
		//panneauAffichage.add("South",scrollAp);
		
		JPanel pLateral = new JPanel();
		LineBorder l = new LineBorder(pLateral.getBackground(),12);
		pLateral.setBorder(l);
		
		pLateral.setLayout(new GridBagLayout());
		c.gridx = 0;
		c.gridy = 0;
		c.insets=new Insets(3,3,3,3);
		pLateral.add(btUndo,c);
		
		c.gridx++;
		pLateral.add(btRedo,c);
		
		c.gridx=0;
		c.ipady=50;
		c.gridy++;
		c.gridwidth = 2;
		pLateral.add(jcbPerkins,c);
		
		c.gridy++;
		c.ipady=10;
		pLateral.add(btEmbosser,c);
		
		c.gridy++;
		c.insets=new Insets(50,3,3,3);
		pLateral.add(btNavigateur,c);
		
		c.gridy++;
		c.insets=new Insets(20,3,3,3);
		pLateral.add(btApercu,c);
		
		/*
		 * Mise en page générale
		 */
		JPanel panneau = new JPanel();
		panneau.setLayout(new BorderLayout());
		panneau.add("North",lFichier);
		panneau.add("Center",panneauAffichage);
		panneau.add("South",lesBoutons);
		panneau.add("East",pLateral);
		JPanel p = new JPanel();
		p.setPreferredSize(new Dimension(10,10));
		panneau.add("West",p);
		setContentPane(panneau);
		if(ConfigNat.getCurrentConfig().getMemoriserFenetre())
		{
			int x= ConfigNat.getCurrentConfig().getWidthEditeur();
			int y=ConfigNat.getCurrentConfig().getHeightEditeur();
			if(x+y != 0){setPreferredSize(new Dimension(x,y));}
		}
		if(ConfigNat.getCurrentConfig().getCentrerFenetre())
		{
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension size = this.getPreferredSize();
			screenSize.height = screenSize.height/2;
			screenSize.width = screenSize.width/2;
			size.height = size.height/2;
			size.width = size.width/2;
			int y = screenSize.height - size.height;
			int x = screenSize.width - size.width;
			setLocation(x, y);
		}
		// ajout action pour saut de page
		/*InputMap inputMap = resultat.getInputMap();
        KeyStroke key1 = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.CTRL_MASK, false);
        inputMap.put(key1, DefaultEditorKit.defaultKeyTypedAction);*/
		initialiseMap();
		encodage=ConfigNat.getCurrentConfig().getSourceEncoding();
        afficheFichier(f, ConfigNat.getCurrentConfig().getPoliceEditeur(), ConfigNat.getCurrentConfig().getTaillePolice());
        jcbPerkins.setSelected(true);
        checkPerkins();
	}
	/** Méthode d'accès, modifie la valeur de l'encodage 
	 * @param enc valeur pour {@link #encodage}*/
	public void setEncodage(String enc){encodage = enc;}
	/** Méthode d'accès, indique le nom de la table braille à utiliser 
	 * @param tb valeur pour {@link #tableBraille}*/
	public void setTableBraille(String tb){tableBraille = tb;}
	
	/** Vérifie l'état du manager undo et active ou désactive les boutons undo/redo */
	private void verifBtEdit() 
	{
		btUndo.setEnabled(undoMng.canUndo());
		btRedo.setEnabled(undoMng.canRedo());
	}
	/** 
	 *	Enregistre le fichier
	 */
	private void enregistrerFichier()
	{
		/* a virer une fois qu'on est sur de filetoolkit
		
		try
		{
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(fichier),encodage));
			bw.write(resultat.getText());
			message.setText("Fichier enregistré");			
			bw.close();
		}
		catch (IOException e)
		{
			System.err.println("erreur dans: " + e);
			message.setText("<html><p color=\"red\">Erreur lors de l'enregistrement</p></html>");
		}*/
		if (FileToolKit.saveStrToFile(getText(), fichier,ConfigNat.getCurrentConfig().getSourceEncoding())){message.setText("Fichier enregistré");}	
		else{message.setText("<html><p color=\"red\">Erreur lors de l'enregistrement</p></html>");}
		repaint();
	}
	
	/**
	 * Affiche le fichier dans le JTextPane resultat et configure la ligne secondaire
	 * @param nomFichier nom du fichier transcrit
	 * @param police police principale
	 * @param taillePolice taille de la police principale
	 * @param police2 police secondaire
	 * @param taillePolice2 taille de la police secondaire
	 */
	public void afficheFichier(String nomFichier,String police, int taillePolice, String police2, int taillePolice2)
	{
		afficheFichier(nomFichier, police, taillePolice);
	}
	
	/**
	 * Affiche le fichier dans le JTextPane
	 * @param nomFichier nom du fichier transcrit
	 * @param police police principale
	 * @param taillePolice taille de la police principale
	 */
	public void afficheFichier(String nomFichier,String police, int taillePolice)
    {
		setTitle("NAT " + nomFichier);
	    fichier = nomFichier;
	    lFichier.setText(lFichier.getText() + fichier);
		//Préparation des styles pour le JTextPane
	    StyledDocument doc = resultat.getStyledDocument();
	    //pour éviter que l'affichage prenne 3 plombes à chaque fois...
	    Document blank = new DefaultStyledDocument();
	    resultat.setDocument(blank);
	    
	    MutableAttributeSet attrs = resultat.getInputAttributes();
	    Font fonteBraille = new Font(police, Font.PLAIN, taillePolice);

	    StyleConstants.setFontFamily(attrs, fonteBraille.getFamily());
	    StyleConstants.setFontSize(attrs,taillePolice);
	    
		resultat.setPreferredSize(new Dimension(400,280));
		apercu.setPreferredSize(new Dimension(400,100));
		this.pack();
		
	    try
	    {
	    	BufferedReader raf = new BufferedReader(new InputStreamReader(new FileInputStream(nomFichier),encodage));
	    	String ligne;
	    	while ( (ligne = raf.readLine()) != null )
	    	{
	    		doc.insertString(doc.getLength(), ligne + "\n", attrs);
	    	}
	     }
	    catch (BadLocationException ble){System.err.println("Impossible d'afficher le texte");}
	    catch (IOException e){System.err.println("erreur dans: " + e);}
			
		initialiseMap();
		
		resultat.setDocument(doc);
		resultat.setCaretPosition(0);
		resultat.getDocument().addDocumentListener(this);
		resultat.getDocument().addUndoableEditListener(this);
    }	  
	
	/**
	 * Affiche en noir dans {@link #apercu} la détranscription du texte de {@link #resultat}
	 */
	private void afficheNoir()
	{
		apercu.removeAll();
		apercu.repaint();
		ArrayList<String>source = new ArrayList<String>();
		ArrayList<String>sortie = new ArrayList<String>();
		source.add(fichier);
		sortie.add("tmp/tmpTan.xhtml");
		if (nat.fabriqueTranscriptions(source, sortie));
	    {			
		   btNavigateur.setEnabled(nat.lanceScenario());
	    }
		//création du DOM
		org.w3c.dom.Document doc;
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setValidating(false);
    
        DocumentBuilder db;
		try
		{
			db = dbf.newDocumentBuilder();
			doc = db.parse(new File( "tmp/tmpTan.xhtml"));
			
			NodeList maths = doc.getElementsByTagName("p");
			MathDocHandler handler = new MathDocHandler();
			ArrayList<MathPanel> lMathPan = new ArrayList<MathPanel>();
			int x=0;
			int y=0;
			for(int i=0;i<maths.getLength();i++)
			{			
				//Création du panneau d'affichage
				MathPanel mp = new MathPanel();
				lMathPan.add(mp);
				apercu.add(mp);
				// contexte graphique
				Graphics2D graph = (Graphics2D) mp.getGraphics();
				graph.setFont( new Font( "serif", Font.PLAIN, ConfigNat.getCurrentConfig().getTaillePolice2()));
				graph.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				
				// Properties des/der Display-Object(s) festlegen
				DispObject.setDefaultColor( graph.getColor());                     // Farbe setzen
				DispObject.setDefaultFont( graph.getFont());                       // Font setzen
				DispObject.setDefaultRenderContext( graph.getFontRenderContext()); // Panel-Kontex
				DispObject.setDefaultSpacing();// einfaches Spacing
				DispObject.term = new StringWriter();                    // Ausgaben in String
				DispObject.out  = new PrintWriter(DispObject.term);     // Ziel der Ausgabe
				
				try
				 {
					TransformerFactory factory = TransformerFactory.newInstance();
					Transformer analyseur = factory.newTransformer();
					DOMSource xmlSrc = new DOMSource(maths.item(i));
					ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
					StreamResult resu = new StreamResult(outputStream);
					analyseur.transform(xmlSrc, resu );
					//System.out.println(outputStream.toString());
					SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
				    parser.parse(new ByteArrayInputStream(outputStream.toString().getBytes()), handler);
			     }
				 catch (Throwable throwable)
			     {
					 throwable.printStackTrace();
				 }
				// Aufbau der Anzeige mit/Ohne Anzeige des Trees
				boolean showtree = false;
				if(ConfigNat.getCurrentConfig().getNiveauLog()==Nat.LOG_DEBUG){showtree=true;}
				if (showtree){System.out.println( "MathML-Tree in postorder:\n");}
				DispObject obj;
				try
				{
					handler.build( handler.tree.root, 0, showtree);
					
					// Anzeigeobjekt auslesen 
					obj = handler.type.result();
					if(obj!=null)
					{
						mp.setObject(obj); // Objekt an Panel binden
					}
				}
				catch (NullPointerException npe)
				{
					obj = new DispString("Erreur de saisie dans "+maths.item(i).getTextContent().trim().replaceAll("[ ]+", " "));
					mp.setObject(obj);
				}

				mp.setBounds(x,y,(int)obj.bnd.getWidth(),(int)obj.bnd.getHeight());
				y = y+(int)obj.bnd.getHeight();
				mp.repaint();
			}
		}
		catch (ParserConfigurationException e) {e.printStackTrace();}//TODO
		catch (SAXException e) {e.printStackTrace();}//TODO
		catch (IOException e) {e.printStackTrace();}//TODO
	
    
	}
	
	
	
	/**
	 * Create the object to be displayed in {@link #apercu}
	 * <p>This method recursively performs displays for each node and node fragments</p>
	 * @param n the current node
	 * @return the {@link formula.translators.DocToDisplay} to display
	 *
	private DispObject createDisplay(Node n)
	{
		DispObject disp=null;
		
		if (n.getNodeType() == Node.TEXT_NODE)
		{
	         disp = new DispString(n.getNodeValue());
	    }
		else if (n instanceof Element) {
	        LinkedList<DispObject> lfils = new LinkedList<DispObject>();
	        NodeList fils = n.getChildNodes();
	        for(int i=0; (i < fils.getLength()-1); i++) {lfils.offer(createDisplay(fils.item(i)));}
	        if(lfils.size()>0){disp = new DispRow(lfils);}
		}

		else if (n instanceof org.w3c.dom.Document)
		{
			NodeList fils = n.getChildNodes();
	        for(int i=0; (i < fils.getLength()); i++) {disp=createDisplay(fils.item(i));}
	    }
		
		return disp;
	}*/
	/**
	 * Initialise la HashMap d'équivalence entre les entités de la forme &pt123456; et leur
	 * valeur en binaire
	 * @return true si succès, false si erreur lors du chargement
	 * @exception NumberFormatException problème lors de la conversion des entités, la table ne doit pas être valide
	 */
	private boolean initialiseMap()
	{
		boolean retour = true;
		//création de la hashtable pt -> caractère 
		String fichierTable = "xsl/tablesBraille/" + tableBraille + ".ent";
		try
		{
			RandomAccessFile raf = new RandomAccessFile(fichierTable, "r");
			String ligne;
			String[] enregistrement;
	      int i=0;
	      
	      ligne = raf.readLine();
	      //on cherche le début des entitées
	      while(!ligne.startsWith("<!ENTITY") && ligne!=null)
	      {
	    	  ligne = raf.readLine();
	      }
	      if (ligne==null)
	      {
	    	  System.err.println("Le fichier" + fichierTable + "n'est pas un fichier valide");
	    	  message.setText("<html><br><p color=\"red\">Le fichier" + fichierTable + "n'est pas un fichier valide</p></html>");
	    	  retour =false;
	      }
			else
			{
				do
				{
					String pt = "0";
					String code ="";
				   enregistrement = ligne.split(" ");
				   if(!enregistrement[2].startsWith("\"&#"))
				   {
					  if (enregistrement[2].startsWith("\"&apos;")){code = "'";}
					  else if (enregistrement[2].startsWith("\"&quot;")){code = "\"";}
					  else if (enregistrement[2].startsWith("\"&lt;")){code = "<";}
					  else if (enregistrement[2].startsWith("\"&gt;")){code = ">";}
					  else {code = "&";}
					}
				   else{ code = Character.toString((char)(Integer.parseInt(enregistrement[2].substring(3, enregistrement[2].length()-3))));}
				   pt = convertitPoint2Int(enregistrement[1].substring(2,enregistrement[1].length()));
				   ptPerkins.put(pt, code);
				   i++;
				}
			   while ((ligne = raf.readLine()) != null && i<64);
			}
			raf.close();
		}
	   catch (IOException e)
	   {
		   System.err.println("erreur dans: " + e);
		   message.setText(":Erreur d'entrée/sortie lors du chargement");
	   }
	   catch (NumberFormatException e)
	   {
		   System.err.println("La table Braille n'est pas valide: " + e);
		   message.setText("<html><br> <p color=\"red\">La table Braille n'est pas valide</p></html>");
	   }
	   return retour;
	}

	/**
	 * Outil de conversion des entités 123456 par la notation "binaire"
	 * @param s La chaine d'origine sous forme 123456
	 * @return Une chaine représentant l'entier en base 10 obtenu par conversion binaire
	 */
	private String convertitPoint2Int(String s)
	{
		int retour = 0;
		if (s.indexOf("1")>= 0){retour = retour + 1;}
		if (s.indexOf("2")>= 0){retour = retour + 2;}
		if (s.indexOf("3")>= 0){retour = retour + 4;}
		if (s.indexOf("4")>= 0){retour = retour + 8;}
		if (s.indexOf("5")>= 0){retour = retour + 16;}
		if (s.indexOf("6")>= 0){retour = retour + 32;}
		return Integer.toString(retour);
	}
	/**
	 * Renvoie le texte contenu dans {@link #resultat}
	 * @return Le texte contenu dans {@link #resultat}
	 */
	public String getText()
	{	
		return resultat.getText();
	}
	
	/** Méthode redéfinie de KeyListener
	 * Gère la saisie en mode Perkins
	 * Gère la navigation
	 * L'affichage est réalisé dans la méthode keyReleased
	 * @param e L'objet KeyEvent intercepté
	 */
	public void keyPressed(KeyEvent e)
	{
		//System.err.println(e.getSource().getClass().getCanonicalName());
		char ch = ' ';
		ch = e.getKeyChar();
		/* PERKINS */
		if(jcbPerkins.isSelected())
		{
			switch (ch)
			{
				case 's':
					tabPoint[3]=true;
					nbTouches++;
				break;
				case 'd':
					tabPoint[4]=true;
					nbTouches++;
				break;
				case 'f':
					tabPoint[5]=true;
					nbTouches++;
				break;
				case 'l':
					tabPoint[0]=true;
					nbTouches++;
				break;
				case 'k':
					tabPoint[1]=true;
				break;
				case 'j':
					tabPoint[2]=true;
					nbTouches++;
				break;	
			}
		}
	}

	/** Méthode redéfinie de KeyListener
	 * Gère la saisie en mode Perkins
	 * Réalise l'affichage du caractère braille dans le JTextPane resultat
	 * @param e L'objet KeyEvent intercepté
	 */
	public void keyReleased(KeyEvent e) 
	{
		char ch = e.getKeyChar();
		if(jcbPerkins.isSelected())
		{			
			if(ch == 's' || ch == 'd' || ch=='f' || ch=='j' || ch=='k' || ch == 'l')
			{
				nbTouches--;
				if(nbTouches <= 0)
				{
					int res =0;
					for (int i=0;i<tabPoint.length;i++)
					{
						if (tabPoint[i])
						{
							res = res + (int)Math.pow(2, tabPoint.length - i -1);
							tabPoint[i] = false;
						}
					}
					//System.err.println("code " + res);
					if (res>0)
					{
						try
						{
							resultat.getDocument().insertString(resultat.getCaretPosition(), ptPerkins.get(Integer.toString(res)), resultat.getInputAttributes());
						}
						catch (Exception exp){}
					}
				}
			}
		}
	}

	/** Méthode redéfinie de KeyListener
	 * ne fait rien
	 * @param e Le KeyEvent
	 */
	public void keyTyped(KeyEvent e){}	
	
	
	/** Méthode redéfinie de DocumentListener
	 * Affiche un message si le test area est modifié
	 * @see javax.swing.event.DocumentListener#insertUpdate(javax.swing.event.DocumentEvent)
	 */
	public void insertUpdate(DocumentEvent de) 
	{
		message.setText("Fichier modifié");
		positionCurseur=  resultat.getCaretPosition();
		verifBtEdit();
	}

	/**
	 * Implémente removeUpdate de DocumentListener
	 * @see javax.swing.event.DocumentListener#removeUpdate(javax.swing.event.DocumentEvent)
	 */
	public void removeUpdate(DocumentEvent de) 
	{
		message.setText("Fichier modifié");		
		verifBtEdit();
	}
	/**
	 * Implémente undoableEditHappened de UndoableEditListener
	 * @see javax.swing.event.UndoableEditListener#undoableEditHappened(javax.swing.event.UndoableEditEvent)
	 */
	public void undoableEditHappened(UndoableEditEvent uee) 
	{
		undoMng.addEdit(uee.getEdit());
		verifBtEdit();
	}
	/**
	 * Implémente focusGained de Focus Listener
	 * positionne le curseur sur le text area
	 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
	 */
	public void focusGained(FocusEvent arg0){resultat.setCaretPosition(positionCurseur);};
	/**
	 * Ne fait rien de plus
	 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
	 */
	public void focusLost(FocusEvent arg0) {positionCurseur=resultat.getCaretPosition();}
	/** 
	 * Implémente la méthode actionPerformed d'ActionListener
	 * Gère les actions des boutons et met à jour l'InputMap du JTextPane resultat en fonction de
	 * l'état du JCheckBox jcbPerkins
	 * @param evt l'objet ActionEvent
	 */
	public void actionPerformed(ActionEvent evt)
	{
		if(evt.getSource()==btEnregistrer)
		{
			enregistrerFichier();
			resultat.grabFocus();
		}
		else if (evt.getSource() == btFermer){this.dispose();}
		else if (evt.getSource() == btUndo && undoMng.canUndo())
		{
			undoMng.undo();
			verifBtEdit();
			resultat.grabFocus();
		}
		else if (evt.getSource() == btRedo && undoMng.canRedo())
		{
			undoMng.redo();
			verifBtEdit();
			resultat.grabFocus();
		}
		else if (evt.getSource()==jcbPerkins){checkPerkins();}
		else if (evt.getSource()==btApercu){afficheNoir();}
		else if (evt.getSource()==btEmbosser)
		{
			String table1 = ConfigNat.getCurrentConfig().getTableBraille();
			String table2 = ConfigNat.getCurrentConfig().getTableEmbossage();
			
			//table brailles fichier transcrit/embossage différentes?
			//UPDATE : curieusement, le textconverter crlf utilisé dans embosseurldc a l'air de ne marcher qu'avec "tmpImp.txt"
			//donc on fait la transfo dans tous les cas.
			/*if(!table1.equals(table2))
			{
				//embosseur.convertiTable();
				//embosseur.Embosser(embosseur.getFichImp());*/
				
				String fichierTemp = "./tmp/sourceText.txt" ;
				String fichImp = "./tmp/tmpImp.txt";
				FileToolKit.saveStrToFile (getText(), fichierTemp);
				FileToolKit.convertBrailleFile(fichierTemp, fichImp,table1,table2,embosseur.getGest());
				embosseur.Embosser(fichImp);
				
			/*}
			else{embosseur.Embosser(fichier);}*/
		}
		else if (evt.getSource()==btNavigateur)
		{
			if(Desktop.getDesktop().isSupported(Desktop.Action.BROWSE))
			{
				
				try 
				{
					File f = new File("tmp/tmpTan.xhtml");
					Desktop.getDesktop().browse(f.toURI());
				}
				catch (IOException e){e.printStackTrace();}
			}
			else
			{
				JOptionPane.showMessageDialog(this,"<html><b>Attention!</b><br>" +
						"Les modifications n'ont pas été enregistrées!<br>" +
						"Continuer le test?","Modifications non chargées",JOptionPane.INFORMATION_MESSAGE);
			}
		}
	}
	/**
	 * Active ou désactive la saisie perkins en fonction de {@link #jcbPerkins}
	 */
	private void checkPerkins()
	{
		if (jcbPerkins.isSelected())
		{
			/* Changement de l'action par défaut pour les touches de la saisie
			 * perkins (s, d, f j, k, l). Amélioration possible: plutôt que d'utiliser
			 * writableAction, fabriquer une Action qui ne fait rien
			 */
			InputMap inputMap = resultat.getInputMap();
	        KeyStroke key = KeyStroke.getKeyStroke('s');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        key = KeyStroke.getKeyStroke('d');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        key = KeyStroke.getKeyStroke('f');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        key = KeyStroke.getKeyStroke('j');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        key = KeyStroke.getKeyStroke('k');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        key = KeyStroke.getKeyStroke('l');
	        inputMap.put(key, DefaultEditorKit.writableAction);
	        
	        resultat.addKeyListener(this);
		}
		else
		{
			/* rétablissement de l'action par défaut associée au touches de la saisie perkins*/
			InputMap inputMap = resultat.getInputMap();
	        KeyStroke key = KeyStroke.getKeyStroke('s');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        key = KeyStroke.getKeyStroke('d');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        key = KeyStroke.getKeyStroke('f');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        key = KeyStroke.getKeyStroke('j');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        key = KeyStroke.getKeyStroke('k');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        key = KeyStroke.getKeyStroke('l');
	        inputMap.put(key, DefaultEditorKit.defaultKeyTypedAction);
	        resultat.removeKeyListener(this);
		}
		resultat.grabFocus();
		
	}
	/**
	 * Ne fait rien de plus
	 * (non-Javadoc)
	 * @see javax.swing.event.DocumentListener#changedUpdate(javax.swing.event.DocumentEvent)
	 */
	@Override
	public void changedUpdate(DocumentEvent arg0) {}
}

