/*
 * NAT - An universal Translator
 * Copyright (C) 2005 Bruno Mascret
 * Contact: bmascret@free.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package ui;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.Collections;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import outils.Embosseur;
import outils.EmbosseurInstalle;
import outils.EmbosseurLDC;
import outils.FileToolKit;

import ui.ConfigurationsComboBoxRenderer;

//*** Gestionnaires ***
import gestionnaires.GestionnaireOuvrir;
import gestionnaires.GestionnaireErreur;
import gestionnaires.GestionnaireOuvrirFenetre;
import nat.ConfigNat;
import nat.Nat;

/**
 * Fenêtre principale de l'interface graphique
 * @author bruno
 *
 */
public class FenetrePrinc extends JFrame implements ActionListener, FocusListener, WindowListener
{
    /** numéro de version pour la sérialisation (inutilisé par NAT) */
    private static final long serialVersionUID = 1L;
    /** JTextField pour l'adresse du fichier source */
    private JTextField  entreeXML = new JTextField("./licence.txt",30);
    /** Label associé à entreeXML
     * @see FenetrePrinc#entreeXML
     * */
    private JLabel lEntree = new JLabel("Source");
    /** Bouton ouvrant le JFileChooser pour l'adresse du fichier source 
     * @see FenetrePrinc#entreeXML
     * @see GestionnaireOuvrir
     * */
    private JButton ouvrirSource = new JButton("Sélectionner source",new ImageIcon("ui/icon/document-open.png"));
    /** JTextField pour l'adresse du fichier de sortie */
    private JTextField  sortie = new JTextField("",30);
    /** Label associé à sortie
    * @see FenetrePrinc#sortie
    * */
    private JLabel lSortie = new JLabel("Sortie");
    /** Bouton ouvrant le JFileChooser pour l'adresse du fichier source 
     * @see FenetrePrinc#sortie
     * @see GestionnaireOuvrir
     * */
    private JButton ouvrirSortie = new JButton("Sélectionner sortie",new ImageIcon("ui/icon/document-open.png"));
    /** Bouton lançant la transcription 
     * @see Nat#fabriqueTranscriptions
     * @see Nat#lanceScenario()
     * */
    private JButton transcrire = new JButton("Transcrire",new ImageIcon("ui/icon/system-run.png"));
    /** Bouton ouvrant l'éditeur de fichier transcrit
     * @see FenetrePrinc#afficheFichier(String adresse)
     */
    private JButton btEditeur = new JButton("Ouvrir le fichier transcrit",new ImageIcon("ui/icon/gtk-edit.png"));
    /** Bouton ouvrant l'éditeur de fichier pré-transcrit
     * @see FenetrePrinc#afficheFichierMep(String adresse)
     * @since 2.0
     */
    private JButton btEditeurMep = new JButton("Ouvrir un fichier à mettre en page",new ImageIcon("ui/icon/document-page-setup.png"));
    /** Bouton ouvrant l'éditeur avec un fichier déjà transcrit
     * @see FenetrePrinc#afficheFichier(String adresse)
     * @since 2.0
     */
    private JButton btEditeurTrans = new JButton("Ouvrir un fichier déjà transcrit",new ImageIcon("ui/icon/gtk-edit.png"));
    /**
     * filte xsl à utiliser pour la transcription
     * @deprecated 2.0
     */
    private JTextField  filtre = new JTextField("xsl.xsl",20);
    /**
     * pour activer le mode débugage
     * <p>Pas encore actif!</p>
     */
    private JCheckBox debug = new JCheckBox();
    /**
     * Afficheur graphique de logs
     * @since 2.0
     */
    private AfficheurJTASwing panneauLog;// = new JTextArea (15,40);

    /**
     * Label associé à jcbConfig
     * @see FenetrePrinc#jcbConfig
     */
    private JLabel lConfig = new JLabel("Configuration active:");
    /** JComboBox listant les configurations possibles  */
    private JComboBox jcbConfig = new JComboBox();
    /**
     * Bouton ouvrant la fenetre d'options
     * @see GestionnaireOuvrirFenetre
     */
    private JButton btOption = new JButton("Options",new ImageIcon("ui/icon/document-properties.png"));
    /**
     * Bouton ouvrant la fenetre d'aide
     * @see GestionnaireOuvrirFenetre
     */
    private JButton btAide = new JButton("Aide",new ImageIcon("ui/icon/help-browser.png"));
    /**
     * Bouton ouvrant la fenetre de renseignements sur Nat
     * @see GestionnaireOuvrirFenetre
     */
    private JButton btAPropos = new JButton("A propos...",new ImageIcon("ui/icon/help-about.png"));
    /** Bouton permettant de quitter le programme */
    private JButton btQuitter = new JButton("Quitter",new ImageIcon("ui/icon/exit.png"));
    /** */
    private JButton btSignalBug = new JButton("Rapporter une erreur",new ImageIcon("ui/icon/script-error.png"));
    
    
    /** JMenu contenant les actions possibles sur la fenêtre principale  */
    private JMenu jmAction = new JMenu("Actions");
    /** JMenu contenant les raccourcis vers les actions d'information de la fenêtre principale  */
    private JMenu jmAPropos = new JMenu("Aide");
    /** JMenuBar de la fenêtre principale */
    private JMenuBar jmb = new JMenuBar();
    /**
     * Elément de menu Choisir source
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrir
     */
    private JMenuItem jmiOuvrirSource;
    /**
     * Elément de menu Choisir cible
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrir
     */
    private JMenuItem jmiOuvrirCible;
    /**
     * Elément de menu Ouvrir options
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrirFenetre
     */
    private JMenuItem jmiOptions;
    /**
     * Elément de menu Transcrire
     * @see Nat#fabriqueTranscriptions
     * @see Nat#lanceScenario()
     */
    private JMenuItem jmiTranscrire;
    /**
     * Elément de menu Ouvrir l'éditeur
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrirFenetre
     */
    private JMenuItem jmiOuvrirTrans;
    /**
     * Elément de menu Quitter
     * @see FenetrePrinc#jmAction
     */
    private JMenuItem jmiQuitter;
    /**
     * Elément de menu Aide
     * @see FenetrePrinc#jmAPropos
     */
    private JMenuItem jmiAide;
    /**
     * Elément de menu A propos de NAT
     * @see FenetrePrinc#jmAPropos
     */
    private JMenuItem jmiAPropos;
    /**
     * Elément de menu Ouvrir fichier à mettre en page
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrir
     */
    private JMenuItem jmiOuvrirMEP;
    /**
     * Elément de menu Ouvrir fichier déjà transcrit
     * @see FenetrePrinc#jmAction
     * @see GestionnaireOuvrir
     */
	private JMenuItem jmiOuvrirDejaTrans;
	/**
	 * Elément de menu Signaler un Bug
	 * @see FenetrePrinc#jmAction
	 */
	private JMenuItem jmiSignalBug;
	/**
	 * Elément de menu détranscription
	 * @see FenetrePrinc#jmiTan
	 */
	private JMenuItem jmiTan;
    
    //private JProgressBar jpb = new JProgressBar();
    /**
     * Indique si le Thread du scénario de transcription est lancé
     */
    private boolean running = false;
   
    /** Le gestionnaire d'erreur de NAT utilisé */
    private GestionnaireErreur gestErreur;
    /** L'instance de Nat */
    private Nat nat;
    /**
     * Vrai si la fenêtre option est ouverte
     * @see GestionnaireOuvrirFenetre
     * @see Configuration
     */
    private boolean optionsOuvertes = false;
	
    /**
     * Constructeur de FenetrePrinc
     * @param n une instance de Nat
     */
    public FenetrePrinc(Nat n)
    {
	    this.setIconImage(new ImageIcon("ui/nat.png").getImage());
	    nat = n;
	    panneauLog = new AfficheurJTASwing(15, 40);
	    panneauLog.setEditable(false);
	    panneauLog.setLineWrap(true);
	    gestErreur = n.getGestionnaireErreur();
	    gestErreur.addAfficheur(panneauLog);
	    /*gestErreur.addAfficheur(new AfficheurConsole());
	    gestErreur.addAfficheur(new AfficheurLog());
	    gestErreur.setModeDebugage(debug.isSelected());*/
	    fabriqueFenetre();
    }

    /** @return @link FenetrePrinc#entreeXML}*/
    public JTextField getEntree(){return entreeXML;}
    /** @return @link FenetrePrinc#filtre}
     * @deprecated 2.0
     * */
    public JTextField getFiltre(){return filtre;}
    /** @return @link FenetrePrinc#sortie}*/
    public JTextField getSortie(){return sortie;}
    /** 
     * Change la valeur du texte de {@link FenetrePrinc#entreeXML}
     * @param entree le nouveau texte
     */
    public void setEntree(String entree){entreeXML.setText(entree);}
    /** 
     * Change la valeur du texte de {@link FenetrePrinc#filtre}
     * @param filtre le nouveau texte
     * @deprecated 2.0
     */
    public void setFiltre(String filtre){this.filtre.setText(filtre);}
    /** 
     * Change la valeur du texte de {@link FenetrePrinc#sortie}
     * et active le bouton {@link FenetrePrinc#btEditeur} si l'adresse donnée est valide
     * @param sortie le nouveau texte
     */
    public void setSortie(String sortie)
    {
    	this.sortie.setText(sortie);
    	this.verifieBtEditeur();
    }
    /** @return {@link FenetrePrinc#optionsOuvertes}*/
    public boolean getOptionsOuvertes(){return optionsOuvertes;}
    /** @param oo la valeur de {@link FenetrePrinc#optionsOuvertes}*/
    public void setOptionsOuvertes(boolean oo){optionsOuvertes=oo;}
	
    /** Fabrique la fenêtre {@link FenetrePrinc} */
    private void fabriqueFenetre()
    {
	    // titre de la fenêtre 
	    this.setTitle("NAT " + nat.getVersionLong());
	    
	    jmiOuvrirSource= new JMenuItem("Sélectionner source");
	    jmiOuvrirSource.setMnemonic('e');
	    jmiOuvrirSource.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiOuvrirSource.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_SOURCE));
	    jmAction.add(jmiOuvrirSource);
		
	    jmiOuvrirCible= new JMenuItem("Sélectionner sortie");
	    jmiOuvrirCible.setMnemonic('s');
	    jmiOuvrirCible.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiOuvrirCible.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_SORTIE));
	    jmAction.add(jmiOuvrirCible);
		
	    jmAction.addSeparator();
		
	    jmiOptions= new JMenuItem("Options");
	    jmiOptions.setMnemonic('o');
	    jmiOptions.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiOptions.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_OPTIONS,this));
	    jmAction.add(jmiOptions);
		
	    jmiTranscrire = new JMenuItem("Transcrire");
	    jmiTranscrire.setMnemonic('t');
	    jmiTranscrire.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiTranscrire.addActionListener(this);
	    jmAction.add(jmiTranscrire);
		
	    jmAction.addSeparator();
		
	    jmiOuvrirTrans = new JMenuItem("Ouvrir la transcription");
	    jmiOuvrirTrans.setMnemonic('l');
	    jmiOuvrirTrans.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiOuvrirTrans.addActionListener(this);
	    jmAction.add(jmiOuvrirTrans);
	    
	    jmiTan = new JMenuItem("Ouvrir l'éditeur de détranscription");
	    jmiTan.setMnemonic('n');
	    jmiTan.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiTan.addActionListener(this);
	    jmAction.add(jmiTan);
		
		jmiOuvrirMEP = new JMenuItem("Ouvrir un fichier à mettre en page");
		jmiOuvrirMEP.setMnemonic('m');
		jmiOuvrirMEP.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		jmiOuvrirMEP.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_MEP));
		jmAction.add(jmiOuvrirMEP);
		
		jmiOuvrirDejaTrans = new JMenuItem("Ouvrir un fichier déjà transcrit");
		jmiOuvrirDejaTrans.setMnemonic('d');
		jmiOuvrirDejaTrans.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		jmiOuvrirDejaTrans.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_TRANS));
		jmAction.add(jmiOuvrirDejaTrans);
		
		jmiSignalBug = new JMenuItem("Rapporter une erreur");
		jmiSignalBug.setMnemonic('p');
		jmiSignalBug.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
		jmiSignalBug.addActionListener(this);
		jmAction.add(jmiSignalBug);
			
		jmAction.addSeparator();
		
	    jmiQuitter = new JMenuItem("Quitter");
	    jmiQuitter.setMnemonic('q');
	    jmiQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiQuitter.addActionListener(this);
	    jmAction.add(jmiQuitter);
		
	    jmiAide = new JMenuItem("Aide");
	    jmiAide.setMnemonic(KeyEvent.VK_F6);
	    jmiAide.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiAide.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_AIDE,this));
	    jmAPropos.add(jmiAide);
		
	    jmAPropos.addSeparator();
		
	    jmiAPropos = new JMenuItem("A propos");
	    jmiAPropos.setMnemonic(java.awt.event.KeyEvent.VK_F11);
	    jmiAPropos.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask(), false));
	    jmiAPropos.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_APROPOS,this));
	    jmAPropos.add(jmiAPropos);
		
	    jmAction.setMnemonic('a');
	    jmAPropos.setMnemonic('i');
		
	    jmb.add(jmAction);
	    jmb.add(jmAPropos);
	    this.add(jmb);
		
	    setName("NAT: fenêtre principale");
	    getAccessibleContext().setAccessibleName("NAT: fenêtre principale");
	    getAccessibleContext().setAccessibleDescription("Transcripteur NAT: Fenêtre principale");
			
	    // fichier d'entrée : label, champ, bouton
	    lEntree.setLabelFor(entreeXML);
	    if(ConfigNat.getCurrentConfig().getFsource().length()>0)
	    {
		    entreeXML.setText(ConfigNat.getCurrentConfig().getFsource());
	    }
	    lEntree.setDisplayedMnemonic('u');
	    entreeXML.setName("Fichier source");
	    entreeXML.getAccessibleContext().setAccessibleName("Champ de saisie du fichier source");
	    entreeXML.getAccessibleContext().setAccessibleDescription("Entrer l'adresse du fichier à transcrire");
	    entreeXML.setToolTipText("Entrer l'adresse du fichier à transcrire (Alt+u)");
	    entreeXML.addFocusListener(this);
	    ouvrirSource.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_SOURCE));
	    ouvrirSource.setMnemonic('e');
	    ouvrirSource.setToolTipText("Sélectionner le fichier à transcrire dans l'arborescence de votre système de fichiers (alt+e) ou (ctrl+e)");
			
	    // fichier de sortie : label, champ, bouton
	    lSortie.setLabelFor(sortie);
	    lSortie.setDisplayedMnemonic('r');
	    if(ConfigNat.getCurrentConfig().getFcible().length()>0)
	    {
		    sortie.setText(ConfigNat.getCurrentConfig().getFcible());
	    }
	    sortie.setName("Fichier sortie");
	    sortie.getAccessibleContext().setAccessibleName("Champ de saisie du fichier cible");
	    sortie.getAccessibleContext().setAccessibleDescription("Entrer l'adresse du fichier transcrit");
	    sortie.setToolTipText("Entrer l'adresse de sauvegarde pour le fichier transcrit (Alt+r)");
	    sortie.addFocusListener(this);
	    ouvrirSortie.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_SORTIE));
	    ouvrirSortie.setMnemonic('s');
	    ouvrirSortie.setToolTipText("Sélectionner le fichier de sortie dans l'arborescence de votre système de fichiers (Alt+s) ou (Ctrl+s)");
	
	
	    // bouton pour lancer la transcription
	    transcrire.addActionListener(this);
	    transcrire.setMnemonic('t');
	    transcrire.getAccessibleContext().setAccessibleName("Bouton transcrire");
	    transcrire.getAccessibleContext().setAccessibleDescription("Valider pour lancer la transcription");
	    transcrire.setToolTipText("Lancer la transcription (Alt+t) ou (Ctrl+t)");
		
	    /*
	      jpb.getAccessibleContext().setAccessibleName("Barre de progression de la transcription");
	      jpb.getAccessibleContext().setAccessibleDescription("Barre de progression indiquant l'état d'avancement de la transcription");
	    */
	    /* c'est quoi ça? 
	     * Bruno: c'est pour cacher / montrer la fenetre de log. C'est pas implémenté: je commente
	 * ********** 
	    voir_log.addActionListener(this);
	    voir_log.setMnemonic('a');
	    voir_log.getAccessibleContext().setAccessibleName("Bouton transcrire2");
	    voir_log.getAccessibleContext().setAccessibleDescription("Valider pou2r lancer la transcription");
	    voir_log.setToolTipText("Lancer la transcri2ption");
	    /* ***********/
	    btEditeur.addActionListener(this);
	    btEditeur.setMnemonic('l');
	    btEditeur.getAccessibleContext().setAccessibleName("Bouton ouvrir le fichier transcrit");
	    btEditeur.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir le fichier transcrit avec l'application choisie");
	    btEditeur.setToolTipText("Ouvrir le fichier transcrit avec l'application choisie (Alt+l) ou (Ctrl+l)");
	    verifieBtEditeur();
		
		btEditeurMep.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_MEP));
		btEditeurMep.setMnemonic('m');
		btEditeurMep.getAccessibleContext().setAccessibleName("Bouton ouvrir un fichier à mettre en page");
		btEditeurMep.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir un fichier à mettre en page");
		btEditeurMep.setToolTipText("Ouvrir un fichier à mettre en page (Alt+m) ou (Ctrl+m)");
		btEditeurMep.setEnabled(false);
		
		
		btEditeurTrans.addActionListener(new GestionnaireOuvrir(this,GestionnaireOuvrir.OUVRIR_TRANS));
		btEditeurTrans.setMnemonic('d');
		btEditeurTrans.getAccessibleContext().setAccessibleName("Bouton ouvrir un fichier déjà transcrit");
		btEditeurTrans.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir un fichier déjà transcrit");
		btEditeurTrans.setToolTipText("Ouvrir un fichier déjà transcrit (Alt+d) ou (Ctrl+d)");
		verifieBtEditeur();
		
		// panneau de log
		debug.setText("Debugage actif");
	
	    JScrollPane scrollLog = new JScrollPane (panneauLog);
	    scrollLog.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	    scrollLog.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
	
	    // bouton pour la fenêtre d'options
	    btOption.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_OPTIONS,this));
	    btOption.setMnemonic('o');
	    btOption.getAccessibleContext().setAccessibleName("Bouton options");
	    btOption.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir la fenêtre de paramétrage");
	    btOption.setToolTipText("Ouvrir la fenêtre de paramétrage (Alt+o)ou (Ctrl+o)");
	
	    // bouton pour la fenêtre d'aide
	    btAide.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_AIDE,this));
	    btAide.setMnemonic(java.awt.event.KeyEvent.VK_F6);
	    btAide.getAccessibleContext().setAccessibleName("Bouton Aide");
	    btAide.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir la fenêtre d'aide de NAT");
	    btAide.setToolTipText("Ouvrir la fenêtre d'aide (Alt+F6) ou (Ctrl+F6)");
	
	    // bouton pour la fenêtre a propos
	    btAPropos.addActionListener(new GestionnaireOuvrirFenetre(GestionnaireOuvrirFenetre.OUVRIR_APROPOS,this));
	    btAPropos.setMnemonic(java.awt.event.KeyEvent.VK_F11);
	    btAPropos.getAccessibleContext().setAccessibleName("Bouton à propos de NAT");
	    btAPropos.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir la fenêtre d'information sur le développement de NAT");
	    btAPropos.setToolTipText("Ouvrir la fenêtre d'information sur NAT (Alt+F11) ou (Ctrl+F11)");
	    
	    // bouton pour la fenêtre de rapport de bug
	    btSignalBug.addActionListener(this);
	    btSignalBug.setMnemonic('p');
	    btSignalBug.getAccessibleContext().setAccessibleName("Bouton rapporter une erreur");
	    btSignalBug.getAccessibleContext().setAccessibleDescription("Valider pour ouvrir la fenêtre de rapport d'erreur");
	    btSignalBug.setToolTipText("Ouvrir la fenêtre de rapport d'erreur (Alt+p) ou (Ctrl+p)");
	
	    // bouton pour quitter
	    btQuitter.addActionListener(this);
	    btQuitter.setMnemonic('q');
	    btQuitter.getAccessibleContext().setAccessibleName("Bouton Quitter NAT");
	    btQuitter.getAccessibleContext().setAccessibleDescription("Valider pour quitter NAT");
	    btQuitter.setToolTipText("Quitter NAT (Alt+q) ou (Ctrl+q) ou (Alt+F4)");
			
	    // label de config
	    lConfig.setLabelFor(jcbConfig);
	    lConfig.setDisplayedMnemonic('c');
	    chargeConfigurations();//jcbConfig);
	    //		jcbConfig.addActionListener(this);
	
	    // combo de choix de la config
	    jcbConfig.setEditable(false);
	    jcbConfig.getAccessibleContext().setAccessibleName("Liste à choix multiples configurations");
	    jcbConfig.getAccessibleContext().setAccessibleDescription("Sélectionner à l'aide des flèches la configuration à utiliser");
	    jcbConfig.setToolTipText("Sélectionner la configuration (Alt+c)");
	    jcbConfig.setRenderer(new ConfigurationsComboBoxRenderer());
	    //	jcbConfig.addActionListener(this);
	
	    setLayout(new GridBagLayout());
	    GridBagConstraints c = new GridBagConstraints();

	    c.insets = new Insets(10,3,3,3);
	    // l 1
	    c.anchor = GridBagConstraints.LINE_END;
	    c.fill = GridBagConstraints.NONE;
	    c.gridx = 0;
	    c.gridy = 1;
	    c.gridwidth = 1;
	    c.weightx = 0.0;
	    add(lEntree,c);
	
	    c.fill = GridBagConstraints.HORIZONTAL;
	    c.anchor = GridBagConstraints.CENTER;
	    c.gridx = 1;
	    c.weightx = 1.0;
	    add(entreeXML,c);
		
	    c.gridx = 2;
	    c.weightx = 0.0;
	    add(ouvrirSource,c);
	
	    // l 2
	    c.anchor = GridBagConstraints.LINE_END;
	    c.fill = GridBagConstraints.NONE;
	    c.gridx = 0;
	    c.gridy++;
	    add(lSortie,c);
	    c.anchor = GridBagConstraints.CENTER;
	    c.fill = GridBagConstraints.HORIZONTAL;
	    c.gridx = 1;
	    add(sortie,c);
	    c.gridx = 2;
	    add(ouvrirSortie,c);
	
		
		// l 3	
		c.gridx = 1;
		c.gridy++;
		add(transcrire,c);
		c.gridx = 2;
		add(btEditeur,c);
		
		// l 3	bis
		c.gridx = 1;
		c.gridy++;
		add(btEditeurMep,c);
		c.gridx = 2;
		add(btEditeurTrans,c);
		
	    // l 4
	    c.anchor = GridBagConstraints.LINE_END;
	    c.fill = GridBagConstraints.NONE;
	    c.gridx = 0;
	    c.gridy++;
	    c.weightx = 0.0;
	    add(lConfig,c);
	
	    c.fill = GridBagConstraints.HORIZONTAL;
	    c.anchor = GridBagConstraints.CENTER;
	    c.gridx = 1;
	    c.weightx = 1.0;
	    add(jcbConfig,c);
		
	    c.gridx = 2;
	    c.weightx = 0.0;
	    add(btOption,c);
		
	    /*/l 4 bis		
	      c.gridx = 1;
	      c.gridy++;
	      add(jpb,c);*/
	
	    // l 5
	    c.gridx = 0;
	    c.gridy++;
	    //	add(voir_log,c);
		
	    // l 6
	    JPanel panneauAffichage = new JPanel();
	    panneauAffichage.add(scrollLog);
	    //JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	    //nom et description devraient appartenir au scrollPane scrollLog ?
	    panneauAffichage.getAccessibleContext().setAccessibleName("Panneau d'affichage");
	    panneauAffichage.getAccessibleContext().setAccessibleDescription("Panneau d'affichage dynamique des traitements (logs)");
	    //	panneauAffichage.setPreferredSize(new Dimension(600,300));
	
	    c.fill = GridBagConstraints.BOTH;
	    c.weightx = 1.0;
	    c.weighty = 1.0;
	    c.gridx = 0;
	    c.gridy++;
	    c.gridwidth = 3;
	    //	add(panneauAffichage,c);
	    //add(splitPane);
	    //c.gridy++;
	    add(scrollLog,c);
	
	    // l 7 (boutons du bas)
	    JPanel pBoutons = new JPanel(new GridBagLayout());
	    
	    c.gridx=0;
	    c.anchor = GridBagConstraints.WEST;
	    c.gridy++;
	    c.weightx = 0.0;
	    c.weighty = 0.0;
	    c.gridwidth = 3;
	    c.insets = new Insets(3,3,3,3);
	    add(pBoutons,c);	    
	    
	    c.anchor = GridBagConstraints.LINE_START;
	    c.gridwidth = 1;
	    c.weightx = 1.0;
	    c.weighty = 1.0;
	    c.gridx=0;
	    c.ipadx=0;
	    c.insets = new Insets(3,20,3,20);
	    pBoutons.add(btAide,c);
	    
	    c.gridx++;
	    pBoutons.add(btAPropos,c);
	    c.gridx++;
	    c.weightx = 2.0;
	    c.weighty = 2.0;
	    pBoutons.add(btSignalBug,c);
	    c.gridx++;
	    pBoutons.add(btQuitter,c);
	    
	    
	    //c'est la méthode de windowslistener qui va gérer la fermeture
	    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    addWindowListener(this);
	    //position de la fenêtre
	    if(ConfigNat.getCurrentConfig().getCentrerFenetre())
        {
		    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		    Dimension size = this.getPreferredSize();
		    screenSize.height = screenSize.height/2;
		    screenSize.width = screenSize.width/2;
		    size.height = size.height/2;
		    size.width = size.width/2;
		    int y = screenSize.height - size.height;
		    int x = screenSize.width - size.width;
		    setLocation(x, y);
        }
    }
	
    /** Ajoute les configurations du répertoire "configurations" dans {@link FenetrePrinc#jcbConfig} 
     * et sélectionne la configuration active.
     */
    public void chargeConfigurations()//JComboBox jcb)
    {
    	jcbConfig.removeActionListener(this);         
        jcbConfig.removeAllItems();                   

        File repertoireSysteme = new File(ConfigNat.getSystemConfigFilterFolder());
        File repertoireUser   = new File(ConfigNat.getUserConfigFilterFolder());   
        ArrayList<File> both = new ArrayList<File>();                        

        try {Collections.addAll(both,repertoireSysteme.listFiles());}
        catch (NullPointerException npe){npe.printStackTrace();}                                                          

        try {Collections.addAll(both, repertoireUser.listFiles());}
        catch (NullPointerException npe){npe.printStackTrace();}                                                        

	File[] listConfs = both.toArray(new File[] {});
	
	ConfigurationsListItem selectedConfig = null;
	for(int i=0;i<listConfs.length;i++)
	    {
	        try
		    {
		        ConfigurationsListItem cli = new ConfigurationsListItem(listConfs[i].getCanonicalPath());
		        if (cli.getIsValid())
			    {
				jcbConfig.addItem(cli);    
				if(cli.getFilename().equals(ConfigNat.getCurrentConfig().getFichierConf())) {
				    selectedConfig = cli;
				}
				
		        }
		    }
		catch (IOException ieo){ieo.printStackTrace();}
	    }
	
	jcbConfig.addActionListener(this);
	if (selectedConfig != null){jcbConfig.setSelectedItem(selectedConfig);}
    }

    /**
     * Ouvre l'éditeur de fichier prêt à la mise en page dans l'éditeur associé, en passant à l'éditeur une instance d' {@link Embosseur}
     * si nécessaire. Utilise l'encoding représenté par {@link nat.OptNames} dans {@link ConfigNat}
     * @param nomFichier Le nom du fichier transcrit à ouvrir
     */
    public void afficheFichierMep(String nomFichier) 
    {
    	gestErreur.afficheMessage("Fonctionalité en développement, non implémentée\n", Nat.LOG_SILENCIEUX);
    	//new EditeurXML(gestErreur, nomFichier);
    	new EditeurXML(gestErreur);
    }
    /**
     * Ouvre l'éditeur de fichier transcrit dans l'éditeur, en passant à l'éditeur une instance d' {@link Embosseur}
     * si nécessaire. Utilise l'encoding représenté par {@link nat.OptNames} dans {@link ConfigNat}
     * @param nomFichier Le nom du fichier transcrit à ouvrir
     */
    public void afficheFichier(String nomFichier) 
    {
	    Embosseur emb = null;
	    if(ConfigNat.getCurrentConfig().getUtiliserCommandeEmbossage())
	    {
		    emb = new EmbosseurLDC(nomFichier,gestErreur);
	    }
	    else if (ConfigNat.getCurrentConfig().getUtiliserEmbosseuse())
	    {
		    emb = new EmbosseurInstalle(nomFichier, gestErreur);
	    }
	    Editeur editeur = new Editeur(ConfigNat.getCurrentConfig().getLongueurLigne(),emb);
	    String sourceEncoding = ConfigNat.getCurrentConfig().getSortieEncoding();
			
	    if (sourceEncoding.equals("automatique"))
	    {
		    sourceEncoding = nat.trouveEncodingSource(nomFichier, gestErreur);
		    if(sourceEncoding == null||sourceEncoding.equals(""))
		    {
			    // pas de bol, on n'a pas réussit à trouver le charset correct
			    sourceEncoding = Charset.defaultCharset().name();
			    gestErreur.afficheMessage("\n** Impossible de détecter l'encodage du fichier sortie.\n** Utilisation de l'encodage par défaut: " + sourceEncoding + "\n",Nat.LOG_NORMAL);
		    }
		    else
		    {
			    gestErreur.afficheMessage("\n** Détection automatique de l'encodage du fichier sortie: "+sourceEncoding+"\n", Nat.LOG_SILENCIEUX);
		    }
	    }
	    editeur.setEncodage(sourceEncoding);
	    editeur.setTableBraille(ConfigNat.getCurrentConfig().getTableBraille());
		
	    editeur.setAfficheLigneSecondaire(ConfigNat.getCurrentConfig().getAfficheLigneSecondaire());
	    editeur.afficheFichier(nomFichier, ConfigNat.getCurrentConfig().getPoliceEditeur(), ConfigNat.getCurrentConfig().getTaillePolice(),ConfigNat.getCurrentConfig().getPolice2Editeur(), ConfigNat.getCurrentConfig().getTaillePolice2());
	    editeur.pack();
	    editeur.setVisible(true);
    }

	
    /** implémentation de actionPerformed(ActionEvent evt) de l'interface ActionListener
     * gère tous les boutons, tous les items des menus, et le changement de configuration par {@link FenetrePrinc#jcbConfig}
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent evt)
    {
	    //lancer transcription
	    if (evt.getSource() == transcrire || evt.getSource() == jmiTranscrire)    
        {
		    //jpb.setValue(jpb.getMinimum());
		    Thread thr = new ThreadJPB();
		    thr.start();
		    if(!ConfigNat.getCurrentConfig().getSaxonAsXsltProcessor())
			{
				gestErreur.afficheMessage(
						"\n***********************************" +
						"* ATTENTION, utilisation de XALAN *" +
						"\n***********************************", Nat.LOG_DEBUG);
				System.setProperty("javax.xml.transform.TransformerFactory",
						"org.apache.xalan.processor.TransformerFactoryImpl");
				//jcbXsltProc.setEnabled(false);
			}
		    else
            {
		    	System.setProperty("javax.xml.transform.TransformerFactory",
				"net.sf.saxon.TransformerFactoryImpl");
            }
			
		    btEditeur.setEnabled(false);
		    this.repaint();
		    if ((entreeXML.getText().length()<1)||(filtre.getText().length()<1)||(sortie.getText().length()<1))
	        {
			    gestErreur.afficheMessage("Entrez un fichier source et un fichier cible", Nat.LOG_SILENCIEUX);
	        }
		    else
		    {
			    panneauLog.setText("");
			    panneauLog.paintImmediately(new Rectangle(0,0,panneauLog.getWidth(),panneauLog.getHeight()));
			    /*gestErreur = new GestionnaireErreur(null,ConfigNat.getCurrentConfig().getNiveauLog());*/
			    //gestErreur.addAfficheur(panneauLog);
			    /*gestErreur.addAfficheur(new AfficheurConsole());
			    gestErreur.addAfficheur(new AfficheurLog());*/
			    gestErreur.setModeDebugage(debug.isSelected());
			    ArrayList<String> sources = new ArrayList<String>();
			    ArrayList<String> sorties = new ArrayList<String>();
			    sources.add(entreeXML.getText());
			    sorties.add(sortie.getText());
			    if (nat.fabriqueTranscriptions(sources, sorties))
			    {			
			    	Thread thrTrans = new ThreadTrans();
				    thrTrans.start();
			    }
		    }
	    }

	    //ouvrir editeur
	    else if (evt.getSource() == btEditeur || evt.getSource() == jmiOuvrirTrans)
	    {
	    	ouvrirEditeur();
	    }
	    //ouvrir editeur tan
	    else if (evt.getSource() == jmiTan)
	    {
	    	ouvrirEditeur(true);
	    }
	    //ouvrir rapport de bug
	    else if (evt.getSource() == btSignalBug || evt.getSource() == jmiSignalBug)
	    {
	    	doBugReport();
	    }
	    //combo box des configurations
	    else if (evt.getSource() == jcbConfig)
	    {			
		    ConfigurationsListItem cli = (ConfigurationsListItem)jcbConfig.getSelectedItem();
		    ConfigNat.charger(cli.getFilename());
	    }
	    else if(evt.getSource() == btQuitter || evt.getSource() == jmiQuitter)
	    {
		    quitter();
	    }
		
    }

    /** 
     * implémentation de focusGained de FocusListener; ne fait rien 
     * @param foc Le FocusEvent
     * */
	public void focusGained(FocusEvent foc){} 
	
	/**
	 * implémentation de focusLost de FocusListener;
	 * positionne le curseur sur le dernier caractère des textes contenus dans {@link #entreeXML} et {@link #sortie}
     * @param foc Le FocusEvent
	 */
    public void focusLost(FocusEvent foc)
    {
	    if (foc.getSource()==sortie)
	    {
		    sortie.setCaretPosition(sortie.getText().length());
		    verifieBtEditeur();
	    }
	    else if (foc.getSource()==entreeXML)
	    {
		    entreeXML.setCaretPosition(entreeXML.getText().length());
	    }
    }
	/** Vérifie si l'adresse contenu dans {@link #sortie} est valide, et si c'est le cas dégrise {@link #btEditeur}*/ 
    private void verifieBtEditeur()
    {
	    if(new File(sortie.getText()).exists()){btEditeur.setEnabled(true);}
	    else{btEditeur.setEnabled(false);}
    }

	/** @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)*/
	public void windowActivated(WindowEvent arg0) {}
	/** implémentation de WindowsListener; quitte le programme
	 * @see #quitter()
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent arg0){quitter();}
	/**@see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)*/
	public void windowClosing(WindowEvent arg0) {}
	/**@see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)*/
	public void windowDeactivated(WindowEvent arg0) {}
	/** @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)*/
	public void windowDeiconified(WindowEvent arg0) {}
	/** @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)*/
	public void windowIconified(WindowEvent arg0) {}
	/**@see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)*/
	public void windowOpened(WindowEvent arg0){}
	
	/** Quitte le programme en enregistrant les options de l'interface graphique et la configuration actuelle*/
	private void quitter()
	{
		gestErreur.afficheMessage("\n** Enregistrement de la configuration de l'interface avant de quitter...", Nat.LOG_NORMAL);
		ConfigNat.getCurrentConfig().setFcible(sortie.getText());
		ConfigNat.getCurrentConfig().setFsource(entreeXML.getText());
		ConfigNat.getCurrentConfig().saveUiConf();
		gestErreur.afficheMessage(" ok! \n Au revoir!" , Nat.LOG_NORMAL);
	    System.exit(0);
	}
	
	/**
	 * Open the TAN editor with an empty doc (documents/nouveau.tan) if tan param set to true,
	 * else open the classic editor
	 * @param tan if true open the TAN editor with an empty doc, else open the classic editor
	 */
	private void ouvrirEditeur(boolean tan)
	{
		if(tan){new EditeurTan(null,"documents/nouveau.tan",nat).setVisible(true);}
		else{ouvrirEditeur();}
	}
	
	/** Ouvre le bon éditeur pour le fichier de sortie en fonction des options choisies */
	private void ouvrirEditeur()
	{
		//utilisation de l'éditeur de nat
		if(ConfigNat.getCurrentConfig().getUseNatEditor())
		{
			if(entreeXML.getText().endsWith(".tan"))
			{
				new EditeurTan(null,entreeXML.getText(),nat).setVisible(true);
			}
			else{afficheFichier(sortie.getText());}
		}
		//utilisation de l'éditeur par défaut de l'environnement
		else if(ConfigNat.getCurrentConfig().getUseDefaultEditor())
		{
	    	Desktop desktop = null;
	        if (Desktop.isDesktopSupported())
	        {
	        	desktop = Desktop.getDesktop();
	        	if (desktop.isSupported(Desktop.Action.OPEN))
	        	{
	        		try{desktop.open(new File(sortie.getText()));}
					catch (IOException e) {gestErreur.afficheMessage("Erreur lors d'entrée-sortie avec le fichier "+ sortie.getText(), Nat.LOG_SILENCIEUX);}
	        	}
	        	else{gestErreur.afficheMessage("Editeur par défaut non trouvé", Nat.LOG_SILENCIEUX);}
	
	        }
	        else{gestErreur.afficheMessage("Bureau non supporté", Nat.LOG_SILENCIEUX);}
		}
		//utilisation d'un éditeur spécifique
		else
		{
			File fsortie = new File(sortie.getText());
			// la tableau cmd contient deux éléments : le chemin de l'éditeur externe et le chemin du fichier à ouvrir.
			String []cmd= new String[2];
			try
			{
	
				cmd[0] = ConfigNat.getCurrentConfig().getEditeur();
				cmd[1] = fsortie.getCanonicalPath();
				Runtime.getRuntime().exec(cmd);
			}
			catch (IOException ioe) {gestErreur.afficheMessage("Erreur d'entrée sortie avec "+ cmd,Nat.LOG_SILENCIEUX); }
		}
	}
	/**
	 * Prépare la fenêtre de dialogue {@link FenetreBugReport} et récupère les informations contenues dans les fichiers
	 * temporaires, les logs, et la sortie
	 */
	private void doBugReport()
	{
		/* 
		 * récupération des informations
		 */
		//fichiers temporaires
		/*String fichiers = "------ fichiers temporaires -------\n";
		File tmp = new File("tmp/tmp.xml");
		if(tmp.exists())
		{
			fichiers = fichiers + "*** tmp.xml ************************************\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath()) 
				+ "\n***********************************************\n";
		}
		tmp = new File("tmp/tmp.xhtml");
		if(tmp.exists())
		{
			fichiers = fichiers + "*** tmp.xhtml ************************************\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath()) 
				+ "\n***********************************************\n";
		}
		tmp = new File("tmp/tmp.mep");
		if(tmp.exists())
		{
			fichiers = fichiers + "*** tmp.mep ************************************\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath()) 
				+ "\n***********************************************\n";
		}
		tmp = new File("tmp/tmp-pass2.txt");
		if(tmp.exists())
		{
			fichiers = fichiers + "*** tmp-pass2.txt ************************************\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath()) 
				+ "\n***********************************************\n";
		}
		// Sortie
		tmp = new File(sortie.getText());
		if(tmp.exists())
		{
			fichiers = fichiers + "------ Sortie ----------------------------\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath())+"\n";
		}
		System.out.println(fichiers);*/
		// Dernier fichier de log
		File tmp = new File("tmp/nat_log.1");
		String fichiers ="";
		if(tmp.exists())
		{
			fichiers = fichiers + "------ LOGS ----------------------------\n"
				+ FileToolKit.loadFileToStr(tmp.getAbsolutePath());
		}
		fichiers = fichiers +  "\n------------ fin -----------------\n";
		
		// lancement de la fenêtre de debug
		FenetreBugReport f= new FenetreBugReport(this,fichiers);
		f.setVisible(true);
		
	}
	/**
	 * Classe interne de {@link FenetrePrinc} permettant de jouer un son à intervalle régulier pendant la transcription
	 */
    private class ThreadJPB extends Thread
    {
	    /**
	     * Joue un son toutes les 5 secondes et le son de fin quand running=false
	     * en fonction des options de l'interface graphique
	     */
	    public void run() 
	    {
	        running=true;
	        AudioClip ac;
	        int cycles = 0;
	        try 
		    {
		        File curDir = new File("");
		        ac = Applet.newAudioClip(new URL("file:"+ curDir.getAbsolutePath()+"/ui/sounds/tic.au"));
		        while(running)
			    {
			        if(cycles%5==0 && ConfigNat.getCurrentConfig().getSonPendantTranscription()){ac.play();}
			        try 
				    {
				        sleep(1000);
				        cycles++;
				    } catch (InterruptedException ie) {ie.printStackTrace();}
					
			    }
		        if(ConfigNat.getCurrentConfig().getSonFinTranscription())
			    {
			        if(gestErreur.getException()==null )
				    {
				        ac = Applet.newAudioClip(new URL("file:"+ curDir.getAbsolutePath()+"/ui/sounds/fin.au"));
				    }
			        else
				    {
				        ac = Applet.newAudioClip(new URL("file:"+ curDir.getAbsolutePath()+"/ui/sounds/erreur.au"));
				    }
			        ac.play();
			    }
		    } catch (MalformedURLException mue) 
		    {
		        mue.printStackTrace();
		    }
			
	    }
    }
    /**
	 * Classe interne de {@link FenetrePrinc} permettant de jouer un son à intervalle régulier pendant la transcription
	 */
    private class ThreadTrans extends Thread
    {
	    /**
	     * Joue un son toutes les 5 secondes et le son de fin quand running=false
	     * en fonction des options de l'interface graphique
	     */
	    public void run() 
	    {
	    	transcrire.setEnabled(false);
	    	nat.lanceScenario();
		    running=false;
		    if (gestErreur.getException() == null )
		    {
			    //mettre en place un gestionnaire
			    if (ConfigNat.getCurrentConfig().getOuvrirEditeur())
			    {
				    //afficheFichier(sortie.getText());
			    	ouvrirEditeur();
			    }
			    btEditeur.setEnabled(true);
			    //jpb.setValue(jpb.getMaximum());
		    }
			transcrire.setEnabled(true);
	    }
    }
    /**
     * Renvoie le gestionanire d'erreur utilisé dnas cette fenêtre 
     * @return le gestionanire d'erreur utilisé
     */
	public GestionnaireErreur getGestErreur() {return gestErreur;}
}
